
"""\nstats.py module

(Requires pstat.py module.)

#################################################
#######  Written by:  Gary Strangman  ###########
#######  Last modified:  July 25, 1998  #########
#################################################

A collection of basic statistical functions for python.  Below are the
function names; type "print stats.mean.__doc__" for info on (e.g.) the mean
function.  The list is obviously incomplete, the functions are all but totally
non-optimized and, although tested, are probably far from bulletproof.  Any
comments and/or additions are welcome (strang@nmr.mgh.harvard.edu).  (A few
additional functions are included, for treasure hunters.)

CENTRAL TENDENCY:  geometricmean
                   harmonicmean
                   mean
                   median
                   medianscore
                   mode

MOMENTS:  moment
          variation
          skewness
          kurtosis

FREQUENCY STATS:  scoreatpercentile
                  percentileofscore
                  histogram
                  cumfreq
                  relfreq

VARIABILITY:  samplevar
              samplestd
              var
              std
              z
              zs

TRIMMING FCNS:  trimboth
                trim1

CORRELATION FCNS:  pearsonr
                   spearmanr
                   pointbiserialr
                   kendalltau
                   linregress

INFERENTIAL STATS:  ttest_1samp
                    ttest_ind
                    ttest_rel
                    chisquare
                    chisquare2d
                    mannwhitneyu
                    ranksums
                    wilcoxsont
                    kruskalwallish
                    friedmanchisquare

PROBABILITY CALCS:  chisqprob
                    zprob
                    fprob
                    betacf
                    betai
                    erfcc
                    gammln
                    binomial (NOT FUNCTIONING)
                    binprob (NOT FUNCTIONING)

ANOVA FUNCTIONS:  F_oneway
                  F_multivar
                  F_nway_btw (NOT FUNCTIONING)
                  F_nway_wi (NOT FUNCTIONING)
                  F_test
                  F_value

SUPPORT FUNCTIONS:  sum
                    cumsum
                    ss
                    summult
                    sumsquared
                    sumdiffsquared
                    shellsort
                    rankdata
                    outputpairedstats
                    findwithin

IF NumPy is installed, most of the above functions will also be defined for
arrays, prepending an 'a' to each name.  Thus, amean() is the function that
will calculate means on array data, etc.
"""


import math, string, sys, pstat, copy, umath
from types import *


##### ALSO ...  assert 0 < x < 1, 'bad x in beta1'
#####           raise ValueError, 'this is the error message'


##########################################################################
########################   LIST-BASED FUNCTIONS   ########################
##########################################################################

### Define these regardless

####################################
#######  CENTRAL TENDENCY  #########
####################################

def geometricmean (inlist):
    """Calculates the geometric mean of the values in the passed list.
That is:  n-th root of (x1 * x2 * ... * xn).  Assumes a '1D' list.

Format:  geometricmean(inlist)\n"""

    mult = 1
    for item in inlist:
	mult = mult * item
    return pow(mult,(1.0/len(inlist)))


def harmonicmean (inlist):
    """Calculates the harmonic mean of the values in the passed list.
That is:  n / (1/x1 + 1/x2 + ... + 1/xn).  Assumes a '1D' list.

Format:  harmonicmean(inlist)\n"""

    sum = 0
    for item in inlist:
	sum = sum + 1.0/item
    return len(inlist) / sum


def mean (inlist):
    """\nReturns the arithematic mean of the values in the passed list.
Assumes a '1D' list, but will function on the 1st dim of an array(!).

Format:  mean(inlist)\n"""

    sum = 0
    for item in inlist:
	sum = sum + item
    return sum/float(len(inlist))


def median (inlist,numbins=1000):
    """\nReturns the computed median value of a list of numbers, given the
number of bins to use for the histogram (more bins brings the computed value
closer to the median score, default number of bins = 1000).  See G.W.
Heiman's Basic Stats (1st Edition), or CRC Probability & Statistics.

Format:  median (inlist, numbins=1000)\n"""

    def cumsum(inlist):
	csum = [0]*len(inlist)
	csum[0] = inlist[0]
	for i in range(1,len(inlist)):
	    csum[i] = csum[i-1] + inlist[i]
	return csum

    binsize = (max(inlist)-min(inlist))/float(numbins)         # calc bin width
    (hist, smallest, size, extras) = histogram(inlist,numbins) # make histogram
    cumhist = cumsum(hist)              # make cumulative histogram
    for i in range(len(cumhist)):	# get 1st(!) index holding 50%ile score
	if cumhist[i]>=len(inlist)/2.0:
	    cfbin = i
	    break
    LRL = smallest + size*cfbin		# get lower read limit of that bin
    cfbelow = cumhist[cfbin-1]
    freq = float(hist[cfbin])		# frequency IN the 50%ile bin
    median = LRL + ((len(inlist)/2.0 - cfbelow)/float(freq))*binsize  # median formula
    return median


def medianscore (inlist):
    """\nReturns the 'middle' score of the passed list.  If there is an even
number of scores, the mean of the 2 middle scores is returned.

Format:  medianscore(inlist)\n"""

    newlist = copy.deepcopy(inlist)
    newlist.sort()
    if len(newlist) % 2 == 0:   # if even number of scores, average middle 2
	index = len(newlist)/2  # integer division correct
	median = float(newlist[index] + newlist[index-1]) /2
    else:
	index = len(newlist)/2  # int divsion gives mid value when count from 0
	median = newlist[index]
    return median


def mode(inlist):
    """\nReturns a list of the modal (most common) score(s) in the passed
list.  If there is more than one such score, all are returned.  The bin-count
for the mode(s) is also returned.

Format:  mode(inlist)
Returns: bin-count for mode(s), a list of modal value(s)\n"""

    scores = pstat.unique(inlist)
    scores.sort()
    freq = []
    for item in scores:
	freq.append(inlist.count(item))
    maxfreq = max(freq)
    mode = []
    stillmore = 1
    while stillmore:
	try:
	    indx = freq.index(maxfreq)
	    mode.append(scores[indx])
	    del freq[indx]
	    del scores[indx]
	except ValueError:
	    stillmore=0
    return maxfreq, mode

####################################
############  MOMENTS  #############
####################################

def moment(inlist,moment=1):
    """\nCalculates the nth moment about the mean for a sample (defaults to
the 1st moment).  Used to calculate coefficients of skewness and kurtosis.

Format:  moment(inlist,moment=1)
Returns: appropriate moment (r) from ... 1/n * SUM((inlist(i)-mean)**r)\n"""

    if moment == 1:
	return mean(inlist)
    else:
	n = len(inlist)
	s = 0
	for x in inlist:
	    s = s + (x-mn)**moment
	return s/float(n)


def variation(inlist):
    """\nReturns the coefficient of variation, as defined in CRC Standard
Probability and Statistics, p.6. TO BE VERIFIED

Format:  variation(inlist)\n"""

    return 100.0*samplestd(inlist)/float(mean(inlist))


def skewness(inlist):
    """\nReturns the coefficient of skewness, as defined in CRC Standard
Probability and Statistics, p.6. TO BE VERIFIED

Format:  skewness(inlist)\n"""

    return moment(inlist,3) / (moment(inlist,2)**1.5)


def kurtosis(inlist):
    """\nReturns the coefficient of skewness, as defined in CRC Standard
Probability and Statistics, p.6. TO BE VERIFIED

Format:  kurtosis(inlist)\n"""

    return moment(inlist,4) / (moment(inlist,2)**2)


def sdescr(inlist):

    n = len(inlist)
    m = mean(inlist)
    sd = std(inlist)
    skew = 0.0
    kurt = 0.0
    for item in inlist:
	skew = skew + pow((item-m)/sd,3)
	kurt = kurt + pow((item-m)/sd,4)
    skew = skew / float(n)
    kurt = kurt / float(n) - 3.0
    return n, m, sd, skew, kurt


####################################
#######  FREQUENCY STATS  ##########
####################################

def scoreatpercentile (inlist, percent):
    """\nReturns the score at a given percentile relative to the distribution
given by inlist.

Format:  scoreatpercentile(inlist,percent)\n"""

    if percent > 1:
	percent = percent / 100.0
    targetcf = percent*len(inlist)
    h, lrl, binsize, extras = histogram(inlist)
    cumhist = cumsum(copy.deepcopy(h))
    for i in range(len(cumhist)):
	if cumhist[i] >= targetcf:
	    break
    score = binsize * ((targetcf - cumhist[i-1]) / float(h[i])) + (lrl+binsize*i)
    return score


def percentileofscore (inlist, score,histbins=10,defaultlimits=None):
    """\nReturns the percentile value of a score relative to the distribution
given by inlist.  Formula depends on the values used to histogram the data(!).

Format:  percentileofscore(inlist,score,histbins=10,defaultlimits=None)\n"""

    h, lrl, binsize, extras = histogram(inlist,histbins,defaultlimits)
    cumhist = cumsum(copy.deepcopy(h))
    i = int((score - lrl)/float(binsize))
    print cumhist[i-1], score, (lrl+binsize*i), binsize, h[i],len(inlist)
    pct = (cumhist[i-1]+((score-(lrl+binsize*i))/float(binsize))*h[i])/float(len(inlist)) * 100
    return pct


def histogram (inlist,numbins=10,defaultreallimits=None):
    """\nReturns (i) a list of histogram bin counts, (ii) the smallest value
of the histogram binning, and (iii) the bin width (the last 2 are not
necessarily integers).  Default number of bins is 10.  If no Lower and
Upper Real Limits are given (as a sequence for the 3rd argument), the routine
picks bins spanning all the numbers in the inlist.

Format:  histogram (inlist, numbins=10, defaultreallimits=None)
Returns: list of bin values, lowerreallimit, binsize, extrapoints\n"""

    if (defaultreallimits <> None):
	if type(defaultreallimits) not in [ListType,TupleType] or len(defaultreallimits)==1: # only one limit given, assumed to be lower one & upper is calc'd
	    lowerreallimit = defaultreallimits
	    upperreallimit = 1.0001 * max(inlist)
	else: # assume both limits given
	    lowerreallimit = defaultreallimits[0]
	    upperreallimit = defaultreallimits[1]
	binsize = (upperreallimit-lowerreallimit)/float(numbins)
    else:     # no limits given for histogram, both must be calc'd
	estbinwidth=(max(inlist)-min(inlist))/float(numbins) + 1 # 1=>cover all
	binsize = ((max(inlist)-min(inlist)+estbinwidth))/float(numbins)
	lowerreallimit = min(inlist) - binsize/2 #lower real limit,1st bin
    bins = [0]*(numbins)
    extrapoints = 0
    for num in inlist:
	try:
	    bintoincrement = int((num-lowerreallimit)/float(binsize))
	    bins[bintoincrement] = bins[bintoincrement] + 1
	except:
	    extrapoints = extrapoints + 1
    if (defaultreallimits <> 0 and extrapoints > 0):
	print
	print 'Points outside given histogram range =',extrapoints
    return (bins, lowerreallimit, binsize, extrapoints)


def cumfreq(inlist,numbins=10,defaultreallimits=None):
    """\nReturns a cumulative frequency histogram, using the histogram function
above.

Format:  cumfreq(inlist,numbins=10,defaultreallimits=None)
Returns: list of cumfreq bin values, lowerreallimit, binsize, extrapoints\n"""

    h,l,b,e = histogram(inlist,numbins,defaultreallimits)
    cumhist = cumsum(copy.deepcopy(h))
    return cumhist,l,b,e


def relfreq(inlist,numbins=10,defaultreallimits=None):
    """\nReturns a cumulative frequency histogram, using the histogram function
above.

Format:  cumfreq(inlist,numbins=10,defaultreallimits=None)
Returns: list of cumfreq bin values, lowerreallimit, binsize, extrapoints\n"""

    h,l,b,e = histogram(inlist,numbins,defaultreallimits)
    for i in range(1,len(h)):
	h[i] = h[i]/float(len(inlist))
    return h,l,b,e


####################################
#####  VARIABILITY FUNCTIONS  ######
####################################

def samplevar (inlist):
    """\nReturns the variance of the values in the passed list using
N for the denominator (i.e., DESCRIBES the sample variance only). Sx**2

Format:  samplevar(inlist)\n"""

    n = len(inlist)
    mn = mean(inlist)
    deviations = [0]*len(inlist)
    for i in range(len(inlist)):
	deviations[i] = inlist[i] - mn
    return ss(deviations)/float(n)


def samplestd (inlist):
    """\nReturns the standard deviation of the values in the passed list using
N for the denominator (i.e., DESCRIBES the sample standard deviation only). Sx

Format:  samplestd(inlist)\n"""

    return math.sqrt(samplevar(inlist))


def var (inlist):
    """\nReturns the variance (sx**2) of the values in the passed list using
N-1 for the denominator (i.e., for estimating population variance).

Format:  var(inlist)\n"""

    n = len(inlist)
    mn = mean(inlist)
    deviations = [0]*len(inlist)
    for i in range(len(inlist)):
	deviations[i] = inlist[i] - mn
    return ss(deviations)/float(n-1)


def std (inlist):
    """\nReturns the standard deviation (sx) of the values in the passed list
using N-1 in the denominator (i.e., to estimate population standard deviation).

Format:  std(inlist)\n"""

    return math.sqrt(var(inlist))


def sem (inlist):
    """\nReturns the estimated standard error of the mean (sx-bar) of the
values in the passed list.  sem = stdev / sqrt(n)

Format:  sem(inlist)\n"""

    sd = std(inlist)
    n = len(inlist)
    return sd/math.sqrt(n)


def z (inlist, score):
    """\nReturns the z-score for a given input score, given that score and the
list from which that score came.  Not appropriate for population calculations.

Format:  z(inlist, score)\n"""

    z = (score-mean(inlist))/samplestd(inlist)
    return z


def zs (inlist):
    """\nReturns a list of z-scores, one for each score in the passed list.

Format:  zs(inlist)"""

    zscores = []
    for item in inlist:
	zscores.append(z(inlist,item))
    return zscores



####################################
#######  TRIMMING FUNCTIONS  #######
####################################

def trimboth (l,proportiontocut):
    """\nSlices off the passed proportion of items from BOTH ends of the passed
list (i.e., 10% of slowest and fastest scores, if the passed list is sorted
by magnitude.  Slices off LESS if proportion results in a non-integer slice
index (i.e., conservatively slices off proportiontocut).

Format:  trimboth (l,proportiontocut)\n"""

    lowercut = int(proportiontocut*len(l))
    uppercut = len(l) - lowercut
    return l[lowercut:uppercut]


def trim1 (l,proportiontocut,tail='right'):
    """\nSlices off the passed proportion of items from ONE end of the passed
list (i.e., 10% of slowest scores, if the passed list is sorted by
increasing magnitude and the tail='right'  Slices off LESS if proportion
results in a non-integer slice index (i.e., conservatively slices off
proportiontocut).

Format:  trim1 (l,proportiontocut,tail='right')\n"""

    if tail == 'right':
	lowercut = 0
	uppercut = len(l) - int(proportiontocut*len(l))
    elif tail == 'left':
	lowercut = int(proportiontocut*len(l))
	uppercut = len(l)
    return l[lowercut:uppercut]


####################################
#####  CORRELATION FUNCTIONS  ######
####################################

def pearsonr(x,y):
    """\nCalculates a Pearson correlation coefficient and the associated
probability value.  Taken from Heiman's Basic Statistics for the Behav.
Sci (2nd), p.195.

Format:  pearsonr(x,y)      where x and y are equal-length lists
Returns: a 2-tuple ... Pearson's r, p-value\n"""

    TINY = 1.0e-30
    if len(x) <> len(y):
	raise ValueError, 'Input values not paired in pearsonr.  Aborting.'
    n = len(x)
    x = map(float,x)
    y = map(float,y)
    xmean = mean(x)
    ymean = mean(y)
    r_num = n*(summult(x,y)) - sum(x)*sum(y)
    r_den = math.sqrt((n*ss(x) - sumsquared(x))*(n*ss(y)-sumsquared(y)))
    r = (r_num / r_den)  # denominator already a float
    df = n-2
    t = r*math.sqrt(df/((1.0-r+TINY)*(1.0+r+TINY)))
    prob = betai(0.5*df,0.5,df/float(df+t*t))
    return r, prob


def spearmanr(x,y):
    """\nCalculates a Spearman rank-order correlation coefficient.  Taken
from Heiman's Basic Statistics for the Behav. Sci (1st), p.192.

Format:  spearmanr(x,y)      where x and y are equal-length lists
Returns: a 2-tuple ... Spearman's r, p-value\n"""

    TINY = 1e-30
    if len(x) <> len(y):
	raise ValueError, 'Input values not paired in spearmanr.  Aborting.'
    n = len(x)
    x = rankdata(x)
    y = rankdata(y)
    dsq = sumdiffsquared(x,y)
    rs = 1 - 6*dsq / float(n*(n**2-1))
    t = rs * math.sqrt((n-2) / ((rs+1.0)*(1.0-rs)))
    df = n-2
    probrs = betai(0.5*df,0.5,df/(df+t*t))  # t already a float
# probability values for rs are from part 2 of the spearman function in
# Numerical Recipies, p.510.  They almost but don't quite jibe with tables.
# @@@ VERIFY
    return rs, probrs


def pointbiserialr(x,y):
    """\nCalculates a point-biserial correlation coefficient and the associated
probability value.  Taken from Heiman's Basic Statistics for the Behav.
Sci (1st), p.194.

Format:  pointbiserialr(x,y)      where x,y are equal-length lists
Returns: a 2-tuple ... Point-biserial r, p-value\n"""

    TINY = 1e-30
    if len(x) <> len(y):
	raise ValueError, 'INPUT VALUES NOT PAIRED IN pointbiserialr.  ABORTING.'
    data = pstat.abut(x,y)
    categories = pstat.unique(x)
    if len(categories) <> 2:
	raise ValueError, "Exactly 2 categories required for pointbiserialr()."
    else:   # there are 2 categories, continue
	codemap = pstat.abut(categories,range(2))
	recoded = pstat.recode(data,codemap,0)
	x = pstat.linexand(data,0,categories[0])
	y = pstat.linexand(data,0,categories[1])
	xmean = mean(pstat.colex(x,1))
	ymean = mean(pstat.colex(y,1))
	n = len(data)
	adjust = math.sqrt((len(x)/float(n))*(len(y)/float(n)))
	rpb = (ymean - xmean)/samplestd(pstat.colex(data,1))*adjust
	df = n-2
	t = rpb*math.sqrt(df/((1.0-rpb+TINY)*(1.0+rpb+TINY)))
	prob = betai(0.5*df,0.5,df/(df+t*t))  # t already a float
	return rpb, prob


def kendalltau(x,y):
    """\nCalculates Kendall's tau ... correlation of ordinal data.  Adapted
from function kendl1 in Numerical Recipies.  @@@ VERIFY

Format:  kendalltau(x,y)
Returns: tau, probability\n"""

    n1 = 0
    n2 = 0
    iss = 0
    for j in range(len(x)-1):
	for k in range(j,len(y)):
	    a1 = x[j] - x[k]
	    a2 = y[j] - x[k]
	    aa = a1 * a2
	    if (aa):             # neither array has a tie
		n1 = n1 + 1
		n2 = n2 + 1
		if aa > 0:
		    iss = iss + 1
		else:
		    iss = iss -1
	    else:
		if (a1):
		    n1 = n1 + 1
		else:
		    n2 = n2 + 1
    tau = iss / math.sqrt(n1*n2)
    svar = (4.0*len(x)+10.0) / (9.0*len(x)*(len(x)-1))
    z = tau / math.sqrt(svar)
    prob = erfcc(abs(z)/1.4142136)
    return tau, prob


def linregress(x,y):
    """\nCalculates a regression line on x,y pairs.  

Format:  linregress(data)      where data is a 2D array of x,y pairs
Returns: slope, intercept, r, prob, sterr-of-estimate\n"""

    TINY = 1.0e-20
    if len(x) <> len(y):
	raise ValueError, 'Input values not paired in linregress.  Aborting.'
    n = len(x)
    x = map(float,x)
    y = map(float,y)
    xmean = mean(x)
    ymean = mean(y)
    r_num = float(n*(summult(x,y)) - sum(x)*sum(y))
    r_den = math.sqrt((n*ss(x) - sumsquared(x))*(n*ss(y)-sumsquared(y)))
    r = r_num / r_den
    z = 0.5*math.log((1.0+r+TINY)/(1.0-r+TINY))
    df = n-2
    t = r*math.sqrt(df/((1.0-r+TINY)*(1.0+r+TINY)))
    prob = betai(0.5*df,0.5,df/(df+t*t))
    slope = r_num / float(n*ss(x) - sumsquared(x))
    intercept = ymean - slope*xmean
    sterrest = math.sqrt(1-r*r)*samplestd(y)
    return slope, intercept, r, prob, sterrest


####################################
#####  INFERENTIAL STATISTICS  #####
####################################

def ttest_1samp(a,popmean,printit=0,name='Sample',writemode='a'):
    """\nCalculates the t-obtained for the independent samples T-test on ONE
group of scores.  Returns t-value, prob.

Format:  ttest_1samp(a,popmean,Name='Sample',printit=0,writemode='a')
Returns: t-value, prob\n"""

    x = mean(a)
    v = var(a)
    n = len(a)
    df = n-1
    svar = ((n-1)*v)/float(df)
    t = (x-popmean)/math.sqrt(svar*(1.0/n))
    prob = betai(0.5*df,0.5,float(df)/(df+t*t))

    if printit <> 0:
	statname = 'Single-sample T-test.'
	outputpairedstats(printit,writemode,
			  'Population','--',popmean,0,0,0,
			  name,n,x,v,min(a),max(a),
			  statname,t,prob)
    return t,prob


def ttest_ind (a, b, printit=0, name1='Samp1', name2='Samp2', writemode='a'):
    """\nCalculates the t-obtained T-test on TWO INDEPENDENT samples of
scores a, and b.  From Numerical Recipies, p.483.  If printit=1, results
are printed to the screen.  If printit='filename', the results are output
to 'filename' using the given writemode (default=append).  Returns t-value,
and prob.

Format:  ttest_ind(a,b,printit=0,name1='Samp1',name2='Samp2',writemode='a')
Returns: t-value, prob\n"""

    x1 = mean(a)
    x2 = mean(b)
    v1 = std(a)**2
    v2 = std(b)**2
    n1 = len(a)
    n2 = len(b)
    df = n1+n2-2
    svar = ((n1-1)*v1+(n2-1)*v2)/float(df)
    t = (x1-x2)/math.sqrt(svar*(1.0/n1 + 1.0/n2))
    prob = betai(0.5*df,0.5,df/(df+t*t))

    if printit <> 0:
	statname = 'Independent samples T-test.'
	outputpairedstats(printit,writemode,
			  name1,n1,x1,v1,min(a),max(a),
			  name2,n2,x2,v2,min(b),max(b),
			  statname,t,prob)
    return t,prob


def ttest_rel (a,b,printit=0,name1='Sample1',name2='Sample2',writemode='a'):
    """\nCalculates the t-obtained T-test on TWO RELATED samples of scores,
a and b.  From Numerical Recipies, p.483.  If printit=1, results are
printed to the screen.  If printit='filename', the results are output to
'filename' using the given writemode (default=append).  Returns t-value,
and prob.

Format:  ttest_rel(a,b,printit=0,name1='Sample1',name2='Sample2',writemode='a')
Returns: t-value, prob\n"""

    if len(a)<>len(b):
	raise ValueError, 'Unequal length arrays in ttest_rel.'
    x1 = mean(a)
    x2 = mean(b)
    v1 = var(a)
    v2 = var(b)
    n = len(a)
    cov = 0
    for i in range(len(a)):
	cov = cov + (a[i]-x1) * (b[i]-x2)
    df = n-1
    cov = cov / float(df)
    sd = math.sqrt((v1+v2 - 2.0*cov)/float(n))
    t = (x1-x2)/sd
    prob = betai(0.5*df,0.5,df/(df+t*t))

    if printit <> 0:
	statname = 'Related samples T-test.'
	outputpairedstats(printit,writemode,
			  name1,n,x1,v1,min(a),max(a),
			  name2,n,x2,v2,min(b),max(b),
			  statname,t,prob)
    return t, prob


def chisquare(f_obs,f_exp=None):
    """\nCalculates a one-way chi square for n samples of scores and returns
the result.  If no expected frequencies are given, the total N is assumed to
be equally distributed across all groups.

Format:  chisquare(f_obs, f_exp=None)
Returns: chisquare-statistic, probability\n"""

    k = len(f_obs)                 # number of groups
    if f_exp == None:
	f_exp = [sum(f_obs)/float(k)] * len(f_obs) # create k bins with = freq.
    chisq = 0
    for i in range(len(f_obs)):
	chisq = chisq + (f_obs[i]-f_exp[i])**2 / float(f_exp[i])
    return chisq, chisqprob(chisq, k-1)


def chisquare2d(contable):
    """\nCalculates a two-way chi square for n samples of scores and returns
the result.  If no expected frequencies are given, the total N is assumed to
be equally distributed across all groups.  Adapted from Numerical Recipies,
p.498.

Format:  chisquare2d(contable)
Returns: chisquare-statistic, df, probability, cramerV, contingency coeff\n"""

    TINY = 1.0e-30
    sum = 0.0
    ni = len(contable)
    nj = len(contable[0])
    nni = len(contable)
    nnj = len(contable[0])
    sumi = [0.0]*ni
    sumj = [0.0]*nj
    for i in range(ni):
	for j in range(nj):
	    sumi[i] = sumi[i] + contable[i][j]
	    sum = sum + contable[i][j]
	if sumi[i] == 0:
	    nni = nni - 1
    for j in range(nj):
	for i in range(ni):
	    sumj[j] = sumj[j] + contable[i][j]
	if sumj[j]==0:
	    nnj = nnj - 1
    df = nni*nnj-nni-nnj+1
    chisq = 0.0
    for i in range(ni):
	for j in range(nj):
	    expctd = sumj[j]*sumi[i] / float(sum)
	    temp = contable[i][j] - expctd
	    chisq = chisq + temp * temp / (expctd + TINY)
    prob = gammq(0.5*df,0.5*chisq)
    minij = min(nni,nnj) - 1
    cramerv = math.sqrt(chisq/(sum*minij))
    ccc = math.sqrt(chisq/(chisq+sum))
    return chisq, df, prob, cramerv, ccc


def mannwhitneyu(x,y):
    """\nCalculates a Mann-Whitney U statistic on the provided scores and
returns the result.  Use only when the n in each condition is < 20 and you
have 2 independent samples of ranks.  REMEMBER: Mann-Whitney U is significant
if the u-obtained is LESS THAN or equal to the critical value of U.

Format:  mannwhitneyu(data)
Returns: u-statistic\n"""

    n1 = len(x)
    n2 = len(y)
    ranked = rankdata(x+y)
    rankx = ranked[0:n1]       # get the x-ranks
    ranky = ranked[n1:]        # the rest are y-ranks
    u1 = n1*n2 + (n1*(n1+1))/2.0 - sum(rankx)
    u2 = n1*n2 + (n2*(n2+1))/2.0 - sum(ranky)
    u = min(u1,u2)
    return u, 1.0 - zprob(u)


def ranksums(x,y):
    """\nCalculates the rank sums statistic on the provided scores and
returns the result.  Use only when the n in each condition is > 20 and you
have 2 independent samples of ranks.  REMEMBER: Mann-Whitney U is significant
if the u-obtained is LESS THAN or equal to the critical value of U.

Format:  ranksums(x,y)
Returns: a z-statistic, crit val=1.96 (or 1.65 for 1-tailed)\n"""

    n1 = len(x)
    n2 = len(y)
    alldata = x+y
    ranked = rankdata(alldata)
    x = ranked[:n1]
    y = ranked[n1:]
    s = sum(x)
    expected = n1*(n1+n2+1) / 2.0
    z = (s - expected) / math.sqrt(n1*n2*(n1+n2+1)/12.0)
    return z, zprob(z)


def wilcoxont(x,y):
    """\nCalculates the Wilcoxon T-test for related samples and returns the
result.  A non-parametric T-test.

Format:  wilcoxont(x,y)
Returns: a t-statistic, 1-tail probability estimate\n"""

    if len(x) <> len(y):
	raise ValueError, 'Unequal N in wilcoxont.  Aborting.'
    d=[]
    for i in range(len(x)):
	diff = x[i] - y[i]
	if diff <> 0:
	    d.append(diff)
    count = len(d)
    absd = map(abs,d)
    absranked = rankdata(absd)
    r_plus = 0
    r_minus = 0
    for i in range(len(absd)):
	if d[i] < 0:
	    r_minus = r_minus + absranked[i]
	else:
	    r_plus = r_plus + absranked[i]
    wt = min(r_plus, r_minus)
    mn = count * (count+1) * 0.25
    se =  math.sqrt(count*(count+1)*(2.0*count+1.0)/24.0)
    z = math.fabs(wt-mn) / se
    return wt, 1.0 - zprob(z)  # 1-tail prob. estimation!!!


def kruskalwallish(*args):
    """\nThe Kruskal-Wallis H-test is a non-parametric ANOVA for 3 or more
groups, requiring at least 5 subjects in each group.  This function
calculates the Kruskal-Wallis H-test for 3 or more independent samples
and returns the result.  

Format:  kruskalwallish(*args)
Returns: H-statistic, probability\n"""

    args = list(args)
    n = [0]*len(args)
    all = []
    n = map(len,args)
    for i in range(len(args)):
	all = all + args[i]
    ranked = rankdata(all)
    for i in range(len(args)):
	args[i] = ranked[0:n[i]]
	del ranked[0:n[i]]
    rsums = []
    for i in range(len(args)):
	rsums.append(sum(args[i])**2)
	rsums[i] = rsums[i] / float(n[i])
    ssbn = sum(rsums)
    totaln = sum(n)
    h = 12.0 / (totaln*(totaln+1)) * ssbn - 3*(totaln+1)
    df = len(args) - 1
    return h, chisqprob(h,df)


def friedmanchisquare(*args):
    """\nFriedman Chi-Square is a non-parametric, one-way within-subjects
ANOVA.  This function calculates the Friedman Chi-square test for repeated
measures and returns the result, along with the associated probability
value.  It assumes 3 or more repeated measures.  Only 3 levels requires a
minimum of 10 subjects in the study.  Four levels requires 5 subjects per
level(??).

Format:  friedmanchisquare(data)
Returns: chi-square statistic, probability\n"""

    args = list(args)
    k = len(args)
    if k < 3:
	raise ValueError, 'Less than 3 levels.  Friedman test not appropriate.'
	return None
    n = len(args[0])
    args = map(rankdata,args)
#@@@    for i in range(n):
#	args[i] = rankdata(args[i])
    ssbn = 0
    for i in range(k):
	ssbn = ssbn + sum(args[i])**2
    chisq = 12.0 / (k*n*(k+1)) * ssbn - 3*n*(k+1)
    return chisq, chisqprob(chisq,k-1)


def signtest(x,y):
    """\nFUNCTION DOES NOT WORK.
Requires implementation of apobin, pobin, which requires a lot ..."""

    raise ValueError, "FUNCTION DOESN'T WORK YET"
    NSIGNTEST = 25          # According to Siegel
    if len(x) <> len(y):
	raise ValueError, 'Unequal n in signtest.  Aborting.'
	return None
    r = N = 0
    for i in range(len(x)):
	if x[i] == y[i]:
	    pass
	else:
	    r = r + 1
	N = N + 1
    if r < N*0.5:
	r = N-r
    if N > NSIGNTEST:
	p = abinprob(N,1,2,r)    # approximation
    else:
	p = binprob(N,1,2,r)     # exact
    return


####################################
####  PROBABILITY CALCULATIONS  ####
####################################

def chisqprob(chisq,df):
    """\nReturns the probability value associated with the provided chi-square
value and df.  Adapted from chisq.c in Gary Perlman's |Stat.

Format:  chisqprob(chisq,df)\n"""

    BIG = 20.0
    def ex(x):
	BIG = 20.0
	if x < -BIG:
	    return 0.0
	else:
	    return math.exp(x)

    if chisq <=0 or df < 1:
	return 1.0
    a = 0.5 * chisq
    if df%2 == 0:
	even = 1
    else:
	even = 0
    if df > 1:
	y = ex(-a)
    if even:
	s = y
    else:
	s = 2.0 * zprob(-math.sqrt(chisq))
    if (df > 2):
	chisq = 0.5 * (df - 1.0)
	if even:
	    z = 1.0
	else:
	    z = 0.5
	if a > BIG:
	    if even:
		e = 0.0
	    else:
		e = math.log(math.sqrt(math.pi))
	    c = math.log(a)
	    while (z <= chisq):
		e = math.log(z) + e
		s = s + ex(c*z-a-e)
		z = z + 1.0
	    return s
	else:
	    if even:
		e = 1.0
	    else:
		e = 1.0 / math.sqrt(math.pi) / math.sqrt(a)
		c = 0.0
		while (z <= chisq):
		    e = e * (a/float(z))
		    c = c + e
		    z = z + 1.0
		return (c*y+s)
    else:
	return s


def erfcc(x):
    """\nReturns the complementary error function erfc(x) with fractional
error everywhere less than 1.2e-7.  Adapted from Numerical Recipies.

Format:  erfcc(x)\n"""

    z = abs(x)
    t = 1.0 / (1.0+0.5*z)
    ans = t * exp(-z*z-1.26551223 + t*(1.00002368+t*(0.37409196+t*(0.9678418+t*(-0.18628806+t*(0.27886807+t*(-1.13520398+t*(1.48851587+t*(-0.82215223+t*0.17087277)))))))))
    if x >= 0:
	return ans
    else:
	return 2.0 - ans


def zprob(z):
    """\nReturns the probability value associated with the provided z-score.
Adapted from z.c in Gary Perlman's |Stat.

Format:  zprob(z)\n"""

    Z_MAX = 6.0    # maximum meaningful z-value
    if z == 0.0:
	x = 0.0
    else:
	y = 0.5 * math.fabs(z)
	if y >= (Z_MAX*0.5):
	    x = 1.0
	elif (y < 1.0):
	    w = y*y
	    x = ((((((((0.000124818987 * w
			-0.001075204047) * w +0.005198775019) * w
		      -0.019198292004) * w +0.059054035642) * w
		    -0.151968751364) * w +0.319152932694) * w
		  -0.531923007300) * w +0.797884560593) * y * 2.0
	else:
	    y = y - 2.0
	    x = (((((((((((((-0.000045255659 * y
			     +0.000152529290) * y -0.000019538132) * y
			   -0.000676904986) * y +0.001390604284) * y
			 -0.000794620820) * y -0.002034254874) * y
		       +0.006549791214) * y -0.010557625006) * y
		     +0.011630447319) * y -0.009279453341) * y
		   +0.005353579108) * y -0.002141268741) * y
		 +0.000535310849) * y +0.999936657524
    if z > 0.0:
	prob = ((x+1.0)*0.5)
    else:
	prob = ((1.0-x)*0.5)
    return prob


def fprob (dfnum, dfden, F):
    """\nReturns the 1-tailed significance level (p-value) of an F
statistic given the degrees of freedom for the numerator (dfR-dfF) and
the degrees of freedom for the denominator (dfF).  It returns the string
'F is less than 1' when appropriate.

Format:  fprob(dfnum, dfden, F)   where usually dfnum=dfbn, dfden=dfwn\n"""

    p = betai(0.5*dfden, 0.5*dfnum, dfden/float(dfden+dfnum*F))
    return p


def betacf(a,b,x):
    """\nThis function evaluates the continued fraction form of the incomplete
Beta function, betai.  (Adapted from: Numerical Recipies in C.)

Format:  betacf(a,b,x)\n"""

    ITMAX = 200
    EPS = 3.0e-7

    bm = az = am = 1.0
    qab = a+b
    qap = a+1.0
    qam = a-1.0
    bz = 1.0-qab*x/qap
    for i in range(ITMAX+1):
	em = float(i+1)
	tem = em + em
	d = em*(b-em)*x/((qam+tem)*(a+tem))
	ap = az + d*am
	bp = bz+d*bm
	d = -(a+em)*(qab+em)*x/((qap+tem)*(a+tem))
	app = ap+d*az
	bpp = bp+d*bz
	aold = az
	am = ap/bpp
	bm = bp/bpp
	az = app/bpp
	bz = 1.0
	if (abs(az-aold)<(EPS*abs(az))):
	    return az
    print 'a or b too big, or ITMAX too small in Betacf.'


def gammln(xx):
    """Returns the gamma function of xx.
    Gamma(z) = Integral(0,infinity) of t^(z-1)exp(-t) dt.
(Adapted from: Numerical Recipies in C.)

Format:  gammln(xx)"""

    coeff = [76.18009173, -86.50532033, 24.01409822, -1.231739516,
	     0.120858003e-2, -0.536382e-5]
    x = xx - 1.0
    tmp = x + 5.5
    tmp = tmp - (x+0.5)*math.log(tmp)
    ser = 1.0
    for j in range(len(coeff)):
	x = x + 1
	ser = ser + coeff[j]/x
    return -tmp + math.log(2.50662827465*ser)


def betai(a,b,x):
    """\nReturns the incomplete beta function:

    I-sub-x(a,b) = 1/B(a,b)*(Integral(0,x) of t^(a-1)(1-t)^(b-1) dt)

where a,b>0 and B(a,b) = G(a)*G(b)/(G(a+b)) where G(a) is the gamma
function of a.  The continued fraction formulation is implemented here,
using the betacf function.  (Adapted from: Numerical Recipies in C.)

Format:  betai(a,b,x)\n"""

    if (x<0.0 or x>1.0): print 'Bad x in Betai'
    if (x==0.0 or x==1.0):
	bt = 0.0
    else:
	bt = math.exp(gammln(a+b)-gammln(a)-gammln(b)+a*math.log(x)+b*
		      math.log(1.0-x))
    if (x<(a+1.0)/(a+b+2.0)):
	return bt*betacf(a,b,x)/float(a)
    else:
	return 1.0-bt*betacf(b,a,1.0-x)/float(b)


def binomial(n,p1,p2,r):

### ugh ... binomial.c in |Stat code

    raise ValueError, "FUNCTION DOESN'T WORK YET"
    maxpow = 0
    maxpow = max(maxpow,n)
    maxpow = max(maxpow,p2)
    return


def binprob(n,p1,p2,r):

    raise ValueError, "FUNCTION DOESN'T WORK YET"
    maxprob = (n*p1) / p2 + 1
    cum = 0.0
    for i in range(r,n+1):
	prob = binomial(n,p1,p2,i)
	if prob == BADPROB:
	    return BADPROB
	if prob == 0.0 and i > maxprob:
	    break
	cum = cum + prob
### ugh ... not done; binomial.c in |Stat code
    return


####################################
#######  ANOVA CALCULATIONS  #######
####################################

def F_oneway(*lists):
    """\nPerforms a 1-way ANOVA, returning an F-value and probability given
any number of groups.  From Heiman, pp.394-7.

Format:  F_oneway (*lists)    where *lists is a set of lists, one per
                                  treatment group\n"""

    a = len(lists)                 # ANOVA on 'a' groups, each in it's own list
    means = [0]*a
    vars = [0]*a
    ns = [0]*a
    alldata = []
    tmp = map(N.array,lists)
    means = map(amean,tmp)
    vars = map(avar,tmp)
    ns = map(len,lists)
    for i in range(len(lists)):
	alldata = alldata + lists[i]
    alldata = N.array(alldata)
    bign = len(alldata)
    sstot = ass(alldata)-(asumsquared(alldata)/float(bign))
    ssbn = 0
    for list in lists:
	ssbn = ssbn + asumsquared(N.array(list))/float(len(list))
    ssbn = ssbn - (asumsquared(alldata)/float(bign))
    sswn = sstot-ssbn
    dfbn = a-1
    dfwn = bign - a
    msb = ssbn/float(dfbn)
    msw = sswn/float(dfwn)
    f = msb/msw
    prob = fprob(dfbn,dfwn,f)
    return f, prob


def F_multivar (data,scorespersubj):
    """\nDoes multivariate ANOVAs.

Format:  F_multivar(data,scorespersubj)    data = |Stat type,
                scorespersubj = how many cols of dependent measures\n"""
    return


def F_nway_btw (data):
    """\nPerforms an n-way ANOVA, returning F-values and probability given
any number of BETWEEN-SUBJECTS factors.  From Heiman, pp.419-32.  Like
|Stat imput (one data point per row).  In the unequal-N case, it does
A-entered first, then B-entered first (Type I ss--Maxwell & Delaney 298-95).
DOESN'T WORK YET!!!!!!!!!!

Format:  F_nway_btw(data)    where 'data' is a list of lists, row=1 RT\n"""

    raise ValueError, "FUNCTION DOESN'T WORK YET"

    print 'F_nway_btw currently uses Type I sums-of-squares.  Type III is better for       non-orthogonal designs.'
    ns = [0]*len(data[0])
    alluniqueslist = [0]*len(data[0])
    for column in range(len(data[0])):   # get Ns and unique vals for ALL cols
	alluniqueslist[column] = pstat.unique(pstat.colex(data,column))
	ns[column] = len(alluniqueslist[column])
    numfactors = len(data[0][1:-1])
    uniquelevels = alluniqueslist[1:-1]  # unique levels in columns 1 to n-1

### CALC SS FOR EACH MAIN EFFECT
    means = [0]*numfactors        # "factors" number of slots for means
    n = [0]*numfactors            # "factors" number of slots for Ns
    sumsqd = [0]*numfactors       # "factors" number of slots for SS
    for factor in range(numfactors):
	numlevels = len(uniquelevels[factor])
	means[factor] = [0]*numlevels  # 1 mean/factor-level
	n[factor] = [0]*numlevels      # 1 N/factor-level
	for level in range(numlevels):
            leveltoget = uniquelevels[factor][level]
	    tmprts = pstat.colex(pstat.linexand(data,factor+1,leveltoget),
				 len(data[0])-1)       # extract block RTs
	    means[factor][level] = mean(tmprts)        # store block mean
	    n[factor][level] = len(tmprts)             # store block N, & SS
	    sumsqd[factor] = sumsqd[factor] + sumsquared(tmprts)/float(len(tmprts))
    allrts = pstat.colex(data,len(data[0])-1)          # get all RTs, SS
    bign = len(allrts)
    sstot = ss(allrts) - sumsquared(allrts)/float(bign)
    ssx = [0]*numfactors                               # calc SSx (x=[A,B,...])
    ssx = N.array(sumsqd) - sumsquared(allrts)/float(bign)
    print

### CALC INTERACTION SSs

    ### THIS FUNCTION DOESN'T WORK
    def interactss(data,prevmeans,interacttype):
	"""interacttype = 2 (for 2-way interacts), 3 for 3-way, etc."""
	numinteractions = N.add.reduce(len(data[1:-1])-interacttype+1)
	for i in range(numinteractions):
	    mns = 0
	    ssi = 0
	return mns, ssi

    ### NONE OF THIS WORKS
    intss = [0]*(numlevels-1)
    mns = []
    for itype in range(2,numlevels+1):
	numinteracts = N.add.reduce(N.arange(numlevels-itype+1)+1)
#	newmns, ssint[itype-2] = interactss(data,itype)
#	mns.append(newmns)

    ### FROM HERE ON IT WORKS FINE (I.E., INTERACTIONS AREN'T IMPLEMENTED YET)
    factordata = pstat.colex(data,range(1,numfactors+1))
    uniquecells = pstat.unique(factordata)        # get unique factor/combos
    cellsqd = [0]*len(uniquecells)
    for cell in range(len(uniquecells)):
	factrts = pstat.abut(factordata,pstat.colex(data,len(data[0])-1))
	tmpdata = pstat.linexand(factrts,range(len(factordata[0])),
				 uniquecells[cell])
	tmprts = pstat.colex(tmpdata,len(tmpdata[0])-1)
#	print "Factor = ", cell, uniquecells[cell], tmprts
	cellsqd[cell] = sumsquared(tmprts)/float(len(tmprts))
    print cellsqd
    ssbn = N.add.reduce(N.array(cellsqd))
    ssbn = ssbn - sumsquared(allrts)/float(bign)
    sswn = sstot - ssbn
    ssint = ssbn - (sum(ssx))

    df = [0]*len(ssx)
    for factor in range(len(ssx)):
	df[factor] = len(uniquelevels[factor])-1
    dfbn = N.multiply.reduce(N.array(df))
    cells = len(uniquecells)
    dfwn = len(data) - cells
    dftot = len(data) - 1
    dfint = N.multiply.reduce(N.array(df))

    print ssx, df, ssint, dfint
    ms = list(N.array(ssx)/N.array(df),'f')
    mswn = sswn / float(dfwn)
    msbn = ssbn / float(dfbn)
    msint = ssint / float(dfint)
    print
    print msint
    print
    ms.append(msint)
    print ms

    print
    print ssx, ssbn, sswn, sstot, df, dfbn, dfwn,dftot, mswn, msbn
    print
    fs = N.array(ms)/mswn
    for f in fs:
	prob = fprob(dfbn,dfwn,f)
	print f, round(prob,5)
    return


def F_nway_wi (data):
    """\nDoes within-subject ANOVA designs.   DOESN'T WORK YET!

Format:  F_nway_wi(data)    data = |Stat format\n"""

    raise ValueError, "FUNCTION DOESN'T WORK YET"

    ns = [0]*len(data[0])
    alluniqueslist = [0]*len(data[0])
    for column in range(len(data[0])):
	alluniqueslist[column] = pstat.unique(pstat.colex(data,column))
	ns[column] = len(alluniqueslist[column])
    numfactors = len(data[0][1:-1])
    uniquelevels = alluniqueslist[1:-1]  # unique levels in columns 1 to n-1

### CALC D-VARIABLES (Maxwell & Delaney pp.622-4)
    coefflist = ((),
		 ((-1,1)),
		 ((-1,0,1),(1,-2,1)),
		 ((-3,-1,1,3),(1,-1,-1,1),(-1,3,-3,1)),
		 ((-2,-1,0,1,2),(2,-1,-2,-1,-2),(-1,2,0,-2,1),(1,-4,6,-4,1)),
		 ((-5,-3,-1,1,3,5),(5,-1,-4,-4,-1,5),(-5,7,4,-4,-7,5),
		  (1,-3,2,2,-3,1),(-1,5,-10,10,-5,1)))
    withinsubj = findwithin(data)
    for factor in range(len(uniquelevels)):
	if withinsubj[factor] == 1:
	    numlevels = len(uniquelevels[factor])    # get num levels of factor
	    coeff = N.array(coeff[numlevels-1])      # get coeff for D-vars
	    tmpdata = [0]*numlevels                  # prepare for data
	    for level in range(numlevels):           # get all data for factor
		rows = linexor(data,factor+1,uniquelevels[factor][level])
		tmpdata[level] = colex(rows,len(rows[0])-1)
	    tmpdata = N.array(tmpdata,'f')
	    newdata = [0]*numlevels-1
	    for level in range(len(coeff)):          # calc D-variables
		newdata[level] = N.add.reduce(tmpdata*coeff[level],1)

    numvariables = numlevels-1
    F_multivar(newdata,numvariables)
    return


def F_test (data):
    """\nDetermines whether data requires a between or within subjects anova,
and then runs the appropriate test.  General form for F_nway_btw and
F_multivar.

Format:  F_test(data)     where data is 1 datum per row.\n"""

    subjects = pstat.unique(pstat.colex(data,0))
    if len(subjects) == len(data):      # fully between if #subj=#datapoints
	F_nway_btw(data)
    else:
	F_nway_wi(data)
    return


def F_value (ER,EF,dfnum,dfden):
    """\nReturns an F-statistic given the following:
	ER  = error associated with the null hypothesis (the Restricted model)
	EF  = error associated with the alternate hypothesis (the Full model)
	dfR-dfF = degrees of freedom of the numerator
	dfF = degrees of freedom associated with the denominator/Full model

Format:  F_value(ER,EF,dfnum,dfden)\n"""

    return ((ER-EF)/float(dfnum) / (EF/float(dfden)))


####################################
########  SUPPORT FUNCTIONS  #######
####################################

def sum (inlist):
    """\nReturns the sum of the items in the passed list.

Format:  sum(inlist)\n"""

    sum = 0
    for item in inlist:
	sum = sum + item
    return sum


def cumsum (inlist):
    """\nReturns a list consisting of the cumulative sum of the items in the
passed list.

Format:  cumsum(inlist)\n"""

    for i in range(1,len(inlist)):
	inlist[i] = inlist[i] + inlist[i-1]
    return inlist


def ss(inlist):
    """\nSquares each value in the passed list, adds up these squares and
returns the result.

Format:  ss(inlist)\n"""

    ss = 0
    for item in inlist:
	ss = ss + item*item
    return ss


def summult (list1,list2):
    """\nMultiplies elements in list1 and list2, element by element, and
returns the sum of all resulting multiplications.

Format:  summult(list1,list2)\n"""

    if len(list1) <> len(list2):
	raise ValueError, "Lists not equal length in summult."
    sum = 0
    for item1,item2 in pstat.abut(list1,list2):
	sum = sum + item1*item2
    return sum


def sumdiffsquared(x,y):
    """\nTakes pairwise differences of the values in lists x and y, squares
these differences, and returns the sum of these squares.

Format:  sumdiffsquared(x,y)
Returns: sum[(x[i]-y[i])**2]\n"""

    sds = 0
    for i in range(len(x)):
	sds = sds + (x[i]-y[i])**2
    return sds


def sumsquared(inlist):
    """\nAdds the values in the passed list, squares the sum, and returns
the result.

Format:  sumsquared(inlist)
Returns: sum(inlist[i]*inlist[i])\n"""

    s = sum(inlist)
    return s*s


def shellsort(inlist):
    """\nShellsort algorithm.  Sorts a 1D-list.

Format:  shellsort(inlist)
Returns: sorted-inlist, sorting-index-vector (for original list)\n"""

    n = len(inlist)
    svec = copy.deepcopy(inlist)
    ivec = range(n)
    gap = n/2   # integer division needed
    while gap >0:
	for i in range(gap,n):
	    for j in range(i-gap,-1,-gap):
		while j>=0 and svec[j]>svec[j+gap]:
		    temp        = svec[j]
		    svec[j]     = svec[j+gap]
		    svec[j+gap] = temp
		    itemp       = ivec[j]
		    ivec[j]     = ivec[j+gap]
		    ivec[j+gap] = itemp
	gap = gap / 2  # integer division needed

#    Now svec is the sorted input vec and ivec has the order
#                        svec[i] = vec[ivec[i]];
#    To convert vec to ranks, march through svec, looking for
#    duplicates, and assign the ranks back to vec.  Duplicate
#    values get the average of their ranks.

    return svec, ivec


def rankdata(inlist):
    """\nRanks the data in inlist, dealing with ties appropritely.  Assumes
a 1D inlist.  Adapted from Gary Perlman's |Stat ranksort.

Format:  rankdata(inlist)
Returns: a list of length equal to inlist, containing rank scores\n"""

    n = len(inlist)
    svec, ivec = shellsort(inlist)
    sumranks = 0
    dupcount = 0
    for i in range(n):
	sumranks = sumranks + i
	dupcount = dupcount + 1
	if i==n-1 or svec[i] <> svec[i+1]:
	    averank = sumranks / float(dupcount) + 1
	    for j in range(i-dupcount+1,i+1):
		inlist[ivec[j]] = averank
	    sumranks = 0
	    dupcount = 0
    return inlist


def outputpairedstats(fname,writemode,name1,n1,m1,se1,min1,max1,name2,n2,m2,se2,min2,max2,statname,stat,prob):
    """\nPrints or write to a file stats for two groups, using the name, n,
mean, sterr, min and max for each group, as well as the statistic name, its
value, and the associated p-value.

Format:  outputpairedstats(fname,writemode,
			   name1,n1,mean1,stderr1,min1,max1,
			   name2,n2,mean2,stderr2,min2,max2,
			   statname,stat,prob)
Returns: None\n"""

    suffix = ''                       # for *s after the p-value
    if  prob < 0.001:  suffix = '  ***'
    elif prob < 0.01:  suffix = '  **'
    elif prob < 0.05:  suffix = '  *'
    title = [['Name','N','Mean','SD','Min','Max']]
    lofl = title+[[name1,n1,round(m1,3),round(math.sqrt(se1),3),min1,max1],
		  [name2,n2,round(m2,3),round(math.sqrt(se2),3),min2,max2]]
    if type(fname)<>StringType or len(fname)==0:
	print
	print statname
	print
	pstat.printcc(lofl)
	print
	print 'Test statistic = ',round(stat,3),'   p = ',round(prob,3),suffix
	print
    else:
	file = open(fname,writemode)
	file.write('\n'+statname+'\n\n')
	file.close()
	pstat.writecc(lofl,fname,'a')
	file = open(fname,'a')
	file.write(pstat.list2string(['\nTest statistic = ',round(stat,4),'   p = ',round(prob,4),suffix,'\n\n']))
	file.close()
    return None


def findwithin (data):
    """\nReturns an integer representing a binary vector, where 1=within-
subject factor, 0=between.  Input equals the entire data array (i.e.,
column 0=random factor, column -1=measured values (those two are skipped).

Format:  findwithin(data)     data in |Stat format\n"""

    numfact = len(data[0])-1
    withinvec = 0
    for col in range(1,numfact):
	examplelevel = pstat.unique(pstat.colex(data,col))[0]
	rows = pstat.linexand(data,col,examplelevel)  # get 1 level of this factor
	factsubjs = pstat.unique(pstat.colex(rows,0))
	allsubjs = pstat.unique(pstat.colex(data,0))
	if len(factsubjs) == len(allsubjs):  # fewer Ss than scores on this factor?
	    withinvec = withinvec + (1 << col)
    return withinvec

def invsquare(num):
        return 1/float(num*num)


#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============
#=============  THE ARRAY-VERSION OF THE STATS FUNCTIONS  ===============

try:                         # DEFINE THESE *ONLY* IF NUMERIC IS AVAILABLE
 import Numeric
 N = Numeric
 import LinearAlgebra
 LA = LinearAlgebra


 class Dispatch:
    def __init__(self, *tuples):
        self._dispatch = {}
        for func, types in tuples:
            for t in types:
                if t in self._dispatch.keys():
                    raise ValueError, "can't have two dispatches on "+str(t)
                self._dispatch[t] = func
        self._types = self._dispatch.keys()

    def __call__(self, arg1, *args, **kw):
        if type(arg1) not in self._types:
            raise TypeError, "don't know how to dispatch %s arguments" %  type(arg1)
        apply(self._dispatch[type(arg1)], (arg1,) + args, kw)

# I'm considering using the dispatch class above to call either the list or
# array version of a function intelligently.  Probably would need to rename
# the list versions of the functions, though.  GS
#
#def lmean(x):
#    print 'calling lmean for', x
#def amean(x):
#    print 'calling amean for', x
#
#mean = Dispatch( (lmean, (ListType, TupleType)),
#                 (amean, (ArrayType,))


#####################################
########  ACENTRAL TENDENCY  ########
#####################################

 def ageometricmean (inarray,dimension=-1):
    """\nCalculates the geometric mean of the values in the passed array.
That is:  n-th root of (x1 * x2 * ... * xn).  Defaults to ALL values in
the passed array.  Use dimension=-1 to flatten array first.  REMEMBER: if
dimension=0, it collapses over dimension 0 ('rows' in a 2D array) only, etc.

Format:  ageometricmean(inarray,dimension=-1)\n"""

    if dimension == -1:
	inarray = N.ravel(inarray)
	size = len(inarray)
	mult = N.multiply.reduce(inarray)
    else:
	size = inarray.shape[dimension]
	mult = N.multiply.reduce(inarray,dimension)
    return N.power(mult,1.0/size)


 def aharmonicmean (inarray,dimension=-1):
    """\nCalculates the harmonic mean of the values in the passed array.
That is:  n / (1/x1 + 1/x2 + ... + 1/xn).  Defaults to ALL values in
the passed array.  Use dimension=-1 to flatten array first.  REMEMBER: if
dimension=0, it collapses over dimension 0 ('rows' in a 2D array) only, etc.

Format:  aharmonicmean(inarray,dimension=-1)\n"""

    if dimension == -1:
	inarray = N.ravel(inarray)
	size = len(inarray)
	sum = N.add.reduce(1.0 / inarray)
    else:
	size = inarray.shape[dimension]
	sum = N.add.reduce(1.0/inarray, dimension)
    return size * sum


 def amean (inarray,dimension=-1):
    """\nCalculates the arithmatic mean of the values in the passed array.
That is:  1/n * (x1 + x2 + ... + xn).  Defaults to ALL values in the
passed array.  Use dimension=-1 to flatten array first.  REMEMBER: if
dimension=0, it collapses over dimension 0 (i.e., collapses over 'rows'
in a 2D array), etc.

Format:  amean(inarray,dimension=-1)\n"""

    if dimension == -1:
	inarray = N.ravel(inarray)
	sum = N.add.reduce(inarray)
	denom = len(inarray)
    else:
	sum = N.add.reduce(inarray,dimension)
	denom = inarray.shape[dimension]
    return sum/float(denom)


 def amedian (inarray,numbins=1000):
    """\nReturns the COMPUTED median value of an array of numbers, given the
number of bins to use for the histogram (more bins approaches finding the
precise median value of the array; default number of bins = 1000).  From
G.W. Heiman's Basic Stats, or CRC Probability & Statistics.
NOTE:  THIS ROUTINE ALWAYS uses the entire passed array (flattens it first).

Format:  amedian(inarray,numbins=1000)\n"""

    inarray = N.ravel(inarray)
    (hist, smallest, binsize, extras) = ahistogram(inarray,numbins)
    cumhist = N.cumsum(hist)            # make cumulative histogram
    otherbins = N.greater_equal(cumhist,len(inarray)/2.0)
    other = list(otherbins)             # list of 0/1s, 1s start at median bin
    cfbin = other.index(1)		# get 1st(!) index holding 50%ile score
    LRL = smallest + binsize*cfbin	# get lower read limit of that bin
    cfbelow = N.add.reduce(hist[0:cfbin])	# cum. freq. below bin
    freq = hist[cfbin]			# frequency IN the 50%ile bin
    median = LRL + ((len(inarray)/2.0-cfbelow)/float(freq))*binsize # MEDIAN
    return median


 def amedianscore (inarray,dimension=-1):
    """\nReturns the 'middle' score of the passed array.  If there is an even
number of scores, the mean of the 2 middle scores is returned.  Can function
with 1D arrays, or on the FIRST dimension of 2D arrays (i.e., dimension can
be -1, to pre-flatten the array, or dimension can equal 0).

Format:  amedianscore(inarray,dimension=-1)\n"""

    if dimension == -1:
	inarray = N.ravel(inarray)
	dimension = 0
    inarray = N.sort(inarray,dimension)
    if inarray.shape[dimension] % 2 == 0:   # if even number of elements
	indx = inarray.shape[dimension]/2   # integer division correct
	median = N.asarray(inarray[indx]+inarray[indx-1]) / 2.0 # @@@asarray?
    else:
	indx = inarray.shape[dimension] / 2 # integer division correct
	median = N.take(inarray,[indx],dimension)
    return median


 def amode(a, dimension=-1):
    """\nReturns a list of the modal (most common) score(s) in the passed
array.  If there is more than one such score, ONLY THE FIRST is returned.
The bin-count for the modal values is also returned.

Format:  amode(a, dimension=-1)
Returns: bin-count for mode(s), a list of modal value(s)\n"""

    scores = pstat.aunique(N.ravel(a))          # get ALL unique values
    olditems = N.zeros(a.shape[not dimension])
    oldcounts = N.zeros(a.shape[not dimension])
    for item in scores:
	template = N.equal(a,item)
	counts = N.add.reduce(template,dimension)
	items = N.where(N.greater(counts,oldcounts),item,olditems)
	oldcounts = N.where(N.greater(counts,oldcounts),counts,oldcounts)
	olditems = items
    return items, oldcounts


#####################################
############  AMOMENTS  #############
#####################################

 def amoment(a,moment=1,dimension=-1):
    """\nCalculates the nth moment about the mean for a sample (defaults to
the 1st moment).  Used to calculate coefficients of skewness and kurtosis.
Can correctly handle only arrays up to 2D.

Format:  amoment(a,moment=1,dimension=-1)
Returns: appropriate moment according to 1/n * SUM((inlist(i)-mean)**r)\n"""

    if dimension == -1:
	a = N.ravel(a)
	dimension = 0
    else:
	a = swapaxes(a,0,dimension)

    if moment == 1:
	mn = 0
    else:
	mn = amean(a,dimension)
	if dimension == 1:              # try killing these
	    mn = mn[:,N.NewAxis]        # two lines (w/ above)
    n = a.shape[dimension]
    s = N.add.reduce(N.power((a-mn),moment),dimension)
    return s / float(n)


 def asdescr(inlist):

    n = len(inlist)
    m = mean(inlist)
    sd = std(inlist)
    skew = 0.0
    kurt = 0.0
    for item in inlist:
	skew = skew + pow((item-m)/sd,3)
	kurt = kurt + pow((item-m)/sd,4)
    skew = skew / float(n)
    kurt = kurt / float(n) - 3.0
    return n, m, sd, skew, kurt


 def avariation(a,dimension=-1):
    """\nReturns the coefficient of variation, as defined in CRC Standard
Probability and Statistics, p.6. TO BE VERIFIED

Format:  avariation(a,dimension=-1)\n"""

    return 100.0*asamplestd(a,dimension)/amean(a,dimension)


 def askewness(a,dimension=-1):
    """\nReturns the coefficient of skewness, as defined in CRC Standard
Probability and Statistics, p.6. TO BE VERIFIED

Format:  askewness(a, dimension=-1)\n"""

    return moment(a,3,dimension) / N.power(moment(a,2,dimension),1.5)


 def akurtosis(a,dimension=-1):
    """\nReturns the coefficient of skewness, as defined in CRC Standard
Probability and Statistics, p.6. TO BE VERIFIED

Format:  akurtosis(a,dimension=-1)\n"""

    return moment(a,4,dimension) / N.power(moment(a,2,dimension),2)


#####################################
######  AFREQUENCY FUNCTIONS  #######
#####################################

 def ascoreatpercentile (inlist, percent):
    """\nReturns the score at a given percentile relative to the distribution
given by inlist.

Format:  ascoreatpercentile(inlist,percent)\n"""

    if percent > 1:
	percent = percent / 100.0
    targetcf = percent*len(inlist)
    h, lrl, binsize, extras = histogram(inlist)
    cumhist = cumsum(copy.deepcopy(h))
    for i in range(len(cumhist)):
	if cumhist[i] >= targetcf:
	    break
    score = binsize * ((targetcf - cumhist[i-1]) / float(h[i])) + (lrl+binsize*i)
    return score


 def apercentileofscore (inlist, score,histbins=10,defaultlimits=None):
    """\nReturns the percentile value of a score relative to the distribution
given by inlist.  Formula depends on the values used to histogram the data(!).

Format:  apercentileofscore(inlist,score,histbins=10,defaultlimits=None)\n"""

    h, lrl, binsize, extras = histogram(inlist,histbins,defaultlimits)
    cumhist = cumsum(copy.deepcopy(h))
    i = int((score - lrl)/float(binsize))
#    print cumhist[i-1], score, (lrl+binsize*i), binsize, h[i],len(inlist)
    pct = (cumhist[i-1]+((score-(lrl+binsize*i))/float(binsize))*h[i])/float(len(inlist)) * 100
    return pct


 def ahistogram (inarray,numbins=10,defaultlimits=[]):
    """\nReturns (i) an array of histogram bin counts, (ii) the smallest value
of the histogram binning, and (iii) the bin width (the last 2 are not
necessarily integers).  Default number of bins is 10.  If no Lower and
Upper Real Limits are given (as a list for the 3rd argument), the
routine picks bins spanning all the numbers in the inarray.  Returns all of
the following:  array of bin values, lowerreallimit, binsize, extrapoints.

Format:  ahistogram(inarray,numbins,defaultlimits=[])\n"""

    inarray = N.ravel(inarray)               # flatten any >1D arrays
    if (defaultlimits <> []):
	lowerreallimit = defaultlimits[0]
	upperreallimit = defaultlimits[1]
	binsize = (upperreallimit-lowerreallimit) / float(numbins)
    else:
	estbinwidth = float(max(inarray)-min(inarray))/float(numbins) + 1
	binsize = (max(inarray)-min(inarray)+estbinwidth)/float(numbins)
	lowerreallimit = min(inarray) - binsize/2.0  #lower real limit,1st bin
    bins = N.zeros(numbins)
    extrapoints = 0
    for num in inarray:
	try:
	    bintoincrement = int((num-lowerreallimit) / float(binsize))
	    bins[bintoincrement] = bins[bintoincrement] + 1
	except:                           # point outside lower/upper limits
	    extrapoints = extrapoints + 1
    if extrapoints > 0:
	print
	print 'Points outside given range =',extrapoints
    return (bins, lowerreallimit, binsize, extrapoints)


#####################################
######  AVARIABILITY FUNCTIONS  #####
#####################################

 def asamplevar (inarray, dimension=-1):
    """\nReturns the standard deviation of the values in the passed array
(i.e., using N).  Defaults to ALL values in the array.  Use dimension=-1
to flatten array first.  Dimensions >=2 DON'T WORK YET!

Format:  asamplevar(inarray,dimension=-1)\n"""

    if dimension == -1:
	inarray = N.ravel(inarray)
	dimension = 0
    if dimension == 1:
	mn = amean(inarray,dimension)[:,N.NewAxis]
    else:
	mn = amean(inarray,dimension)
    deviations = inarray - mn 
    svar = ass(deviations,dimension) / float(inarray.shape[dimension])
    return svar


 def asamplestd (inarray, dimension=-1):
    """\nReturns the sample 0standard deviation of the values in the passed
array (i.e., using N).  Defaults to ALL values in the array.  Use dimension=-1
to flatten array first.  Dimensions >=1 DON'T WORK YET!

Format:  asamplestd(inarray,dimension=-1)\n"""

    return N.sqrt(asamplevar(inarray,dimension))


 def avar (inarray, dimension=-1):
    """\nReturns the estimated population variance of the values
in the passed array (i.e., N-1).  Defaults to ALL values in the array.
Use dimension=-1 to flatten array first.  Dimensions >=2 DON'T WORK YET!

Format:  avar(inarray,dimension=-1)\n"""

    if dimension == -1:
	inarray = N.ravel(inarray)
	dimension = 0
    if dimension == 1:
	mn = amean(inarray,dimension)[:,N.NewAxis]
    else:
	mn = amean(inarray,dimension)
    deviations = inarray - mn
    var = ass(deviations,dimension)/float(inarray.shape[dimension]-1)
    return var


 def astd (inarray, dimension=-1):
    """\nReturns the estimated population variance of the values
in the passed array (i.e., N-1).  Defaults to ALL values in the array.
Use dimension=-1 to flatten array first.  Dimensions >=1 DON'T WORK YET!

Format:  astd(inarray,dimension=-1)\n"""

    return N.sqrt(avar(inarray,dimension))


 def asem (inarray, dimension=-1):
    """\nReturns the standard error of the mean (i.e., using N) of the
values in the passed array.  Defaults to ALL values in the array.  Use
dimension=-1 to flatten array first.  Dimensions >=1 DON'T WORK YET!

Format:  asem(inarray,dimension=-1)\n"""

    if dimension == -1:
	inarray = N.ravel(inarray)
	dimension = 0
    s = asamplestd(inarray,dimension) / N.sqrt(inarray.shape[dimension]-1)
    return s


 def az (a, score):
    """\nReturns the z-score for a given input score, given that score and the
list from which that score came.  Not appropriate for population calculations,
nor for arrays > 1D.

Format:  az(a, score)\n"""

    z = (score-amean(a)) / asamplestd(a)
    return z


 def azs (a):
    """\nReturns an array of z-scores, one for each score in the passed array.

Format:  azs(a)\n"""

    zscores = []
    for item in a:
	zscores.append(z(a,item))
    return N.array(zscores)


#####################################
#######  ATRIMMING FUNCTIONS  #######
#####################################

 def atrimboth (l,proportiontocut):
    """\nSlices off the passed proportion of items from BOTH ends of the passed
list (i.e., 10% of slowest and fastest scores, if the passed list is sorted
by magnitude.  Slices off LESS if proportion results in a non-integer slice
index (i.e., conservatively slices off proportiontocut).

Format:  atrimboth (l,proportiontocut)\n"""

    lowercut = int(proportiontocut*len(l))
    uppercut = len(l) - lowercut
    return l[lowercut:uppercut]


 def atrim1 (l,proportiontocut,tail='right'):
    """\nSlices off the passed proportion of items from ONE end of the passed
list (i.e., 10% of slowest scores, if the passed list is sorted by
increasing magnitude and the tail='right'  Slices off LESS if proportion
results in a non-integer slice index (i.e., conservatively slices off
proportiontocut).

Format:  atrim1 (l,proportiontocut,tail='right')\n"""

    if tail == 'right':
	lowercut = 0
	uppercut = len(l) - int(proportiontocut*len(l))
    elif tail == 'left':
	lowercut = int(proportiontocut*len(l))
	uppercut = len(l)
    return l[lowercut:uppercut]


#####################################
#####  ACORRELATION FUNCTIONS  ######
#####################################

 def apearsonr(x,y):
    """\nCalculates a Pearson correlation coefficient and returns p.  Taken
from Heiman's Basic Statistics for the Behav. Sci (2nd), p.195.

Format:  apearsonr(x,y)      where x,y are equal length arrays\n"""

    TINY = 1.0e-20
    n = len(x)
    xmean = amean(x)
    ymean = amean(y)
    r_num = n*(N.add.reduce(x*y)) - N.add.reduce(x)*N.add.reduce(y)
    r_den = math.sqrt((n*ass(x) - asumsquared(x))*(n*ass(y)-asumsquared(y)))
    r = (r_num / r_den)[0]
    df = n-2
    t = r*math.sqrt(df/((1.0-r+TINY)*(1.0+r+TINY)))
    prob = betai(0.5*df,0.5,df/(df+t*t))
    return r,prob


 def aspearmanr(x,y):
    """\nCalculates a Spearman rank-order correlation coefficient.  Taken
from Heiman's Basic Statistics for the Behav. Sci (1st), p.192.

Format:  aspearmanr(x,y)      where x,y are equal-length arrays
Returns: a 2-tuple ... Spearman's r, p-value\n"""

    TINY = 1e-30
    n = len(x)
    dsq = N.add.reduce((x-y)**2)
    rs = 1 - 6*dsq / float(n*(n**2-1))
    t = rs * math.sqrt((n-2) / ((rs+1.0)*(1.0-rs)))
    df = n-2
    probrs = betai(0.5*df,0.5,df/(df+t*t))
# probability values for rs are from part 2 of the spearman function in
# Numerical Recipies, p.510.  They almost but don't quite(?) jibe with tables.
    return rs, probrs


 def apointbiserialr(x,y):
    """\nCalculates a point-biserial correlation coefficient and the associated
probability value.  Taken from Heiman's Basic Statistics for the Behav.
Sci (1st), p.194.

Format:  apointbiserialr(x,y)      where x,y are equal length arrays
Returns: a 2-tuple ... Point-biserial r, p-value\n"""

    TINY = 1e-30
    categories = pstat.aunique(x)
    data = pstat.aabut(x,y)
    if len(categories) <> 2:
	raise ValueError, "Exactly 2 categories required for pointbiserialr()."
    else:   # there are 2 categories, continue
	codemap = pstat.abut(categories,range(2))
	recoded = pstat.arecode(data,codemap,0)
	x = pstat.alinexand(data,0,categories[0])
	y = pstat.linexand(data,0,categories[1])
	xmean = amean(pstat.colex(x,1))
	ymean = amean(pstat.colex(y,1))
	n = len(data)
	adjust = math.sqrt((len(x)/float(n))*(len(y)/float(n)))
	rpb = (ymean - xmean)/asamplestd(pstat.acolex(data,1))*adjust
	df = n-2
	t = rpb*math.sqrt(df/((1.0-rpb+TINY)*(1.0+rpb+TINY)))
	prob = betai(0.5*df,0.5,df/(df+t*t))
	return rpb, prob


 def alinregress(x,y):
    """\nCalculates a regression line on an array of x,y pairs.

Format:  alinregress(x,y)      where x,y are equal length arrays
Returns: slope, intercept, r, prob, sterrest\n"""

    TINY = 1.0e-20
    n = len(x)
    xmean = amean(x)
    ymean = amean(y)
    r_num = n*(N.add.reduce(x*y)) - N.add.reduce(x)*N.add.reduce(y)
    r_den = math.sqrt((n*ass(x) - asumsquared(x))*(n*ass(y)-asumsquared(y)))
    r = r_num / r_den
    r = r[0]
    z = 0.5*math.log((1.0+r+TINY)/(1.0-r+TINY))
    df = n-2
    t = r*math.sqrt(df/((1.0-r+TINY)*(1.0+r+TINY)))
    prob = betai(0.5*df,0.5,df/(df+t*t))
    slope = r_num / (float(n)*ass(x) - asumsquared(x))
    intercept = ymean - slope*xmean
    sterrest = math.sqrt(1-r*r)*asamplestd(y)
    return slope, intercept, r, prob, sterrest


 def akendalltau(x,y):
    """\nCalculates Kendall's tau ... correlation of ordinal data.  Adapted
from function kendl1 in Numerical Recipies.  TO BE VERIFIED

Format:  akendalltau(x,y)
Returns: tau, probability\n"""

    n1 = 0
    n2 = 0
    iss = 0
    for j in range(len(x)-1):
	for k in range(j,len(y)):
	    a1 = x[j] - x[k]
	    a2 = y[j] - x[k]
	    aa = a1 * a2
	    if (aa):             # neither array has a tie
		n1 = n1 + 1
		n2 = n2 + 1
		if aa > 0:
		    iss = iss + 1
		else:
		    iss = iss -1
	    else:
		if (a1):
		    n1 = n1 + 1
		else:
		    n2 = n2 + 1
    tau = iss / math.sqrt(n1*n2)
    svar = (4.0*len(x)+10.0) / (9.0*len(x)*(len(x)-1))
    z = tau / math.sqrt(svar)
    prob = erfcc(abs(z)/1.4142136)
    return tau, prob


#####################################
#####  AINFERENTIAL STATISTICS  #####
#####################################

 def attest_1samp(a,popmean,printit=0,name='Sample',writemode='a'):
    """\nCalculates the t-obtained for the independent samples T-test on ONE
group of scores.  Returns t-value, prob.

Format:  attest_1samp(a,popmean,Name='Sample',printit=0,writemode='a')
Returns: t-value, p-value\n"""

    if type(a) != N.ArrayType:
	a = N.array(a)
    x = amean(a)
    v = avar(a)
    n = len(a)
    df = n-1
    svar = ((n-1)*v) / float(df)
    t = (x-popmean)/math.sqrt(svar*(1.0/n))
    prob = betai(0.5*df,0.5,df/(df+t*t))

    if printit <> 0:
	statname = 'Single-sample T-test.'
	outputpairedstats(printit,writemode,
			  'Population','--',popmean,0,0,0,
			  name,n,x,v,min(N.ravel(a)),max(N.ravel(a)),
			  statname,t,prob)
    return t,prob


 def attest_ind (a, b, dimension=-1, printit=0, name1='Samp1', name2='Samp2',writemode='a'):
    """\nCalculates the t-obtained T-test on TWO INDEPENDENT samples of
scores a, and b.  From Numerical Recipies, p.483.  If printit=1, results
are printed to the screen.  If printit='filename', the results are output
to 'filename' using the given writemode (default=append).  Returns t-value,
and prob.

Format:  attest_ind (a,b,Name1='Samp1',Name2='Samp2',printit=0,writemode='a')\n"""

    if dimension == -1:
	a = N.ravel(a)
	b = N.ravel(b)
	dimension = 0
    x1 = amean(a,dimension)
    x2 = amean(b,dimension)
    v1 = avar(a,dimension)
    v2 = avar(b,dimension)
    n1 = a.shape[0]
    n2 = b.shape[0]
    df = n1+n2-2
    svar = ((n1-1)*v1+(n2-1)*v2) / float(df)
    t = (x1-x2)/N.sqrt(svar*(1.0/n1 + 1.0/n2))
    probs = N.zeros(N.ravel(t).shape,'f')
    for i in range(len(N.ravel(t))):
	probs[i] = betai(0.5*df,0.5,float(df)/(df+N.ravel(t)[i]*N.ravel(t)[i]))
    if type(t) == N.ArrayType:
	probs = N.reshape(probs,t.shape)

    if printit <> 0:
	statname = 'Independent samples T-test.'
	outputpairedstats(printit,writemode,
			  name1,n1,x1,v1,min(N.ravel(a)),max(N.ravel(a)),
			  name2,n2,x2,v2,min(N.ravel(b)),max(N.ravel(b)),
			  statname,t,prob)
	return
    return t, probs


 def attest_rel (a,b,printit=0,name1='Samp1',name2='Samp2',writemode='a'):
    """\nCalculates the t-obtained T-test on TWO RELATED samples of scores,
a and b.  From Numerical Recipies, p.483.  If printit=1, results are
printed to the screen.  If printit='filename', the results are output to
'filename' using the given writemode (default=append).  Returns t-value,
and prob.

Format:  attest_rel(a,b,printit=0,name1='Samp1',name2='Samp2',writemode='a')\n"""

    if len(a)<>len(b):
	raise ValueError, 'Unequal length arrays.'
    x1 = amean(a)
    x2 = amean(b)
    v1 = avar(a)
    v2 = avar(b)
    n = len(a)
    cov = N.add.reduce((a-x1)*(b-x2))
    df = n-1
    cov = cov / float(df)
    sd = math.sqrt((v1+v2 - 2.0*cov) / float(n))
    t = (x1-x2) / sd
    prob = betai(0.5*df,0.5,df/(df+t*t))
    if printit <> 0:
	statname = 'Related samples T-test.'
	outputpairedstats(printit,writemode,
			  name1,n,x1,v1,min(N.ravel(a)),max(N.ravel(a)),
			  name2,n,x2,v2,min(N.ravel(b)),max(N.ravel(b)),
			  statname,t,prob)
	return
    return t, prob


 def attest_image (a, b):
    """\nCalculates the t-obtained for the independent samples T-test on the
two stacks of images a, and b.  From Numerical Recipies, p.483.  PROBABLY
OBSOLETE, THANKS TO A BETTER ATTEST_IND, ABOVE.

Format:  attest_image (a,b)   where a,b are [time,x,y] 3D arrays\n"""

    x1 = N.divide(N.add.reduce(a,0),float(a.shape[0]))
    x2 = N.divide(N.add.reduce(b,0),float(b.shape[0]))
    v1 = avar(a,0)
    v2 = avar(b,0)
    n1 = a.shape[0]
    n2 = b.shape[0]
    df = n1+n2-2
    svar = ((n1-1)*v1+(n2-1)*v2) / float(df)
    t = (x1-x2)/N.sqrt(svar*(1.0/n1 + 1.0/n2))
    probs = N.zeros(N.ravel(t).shape,'f')
    for i in range(len(probs)):
	probs[i] = betai(0.5*df,0.5,float(df)/(df+t.flat[i]*t.flat[i]))
    return N.reshape(probs,x1.shape)


 def achisquare(f_obs,f_exp=None):
    """\nCalculates the chi-square-obtained given the observed frequencies
in a 1-way test.  If no expected frequencies are given, the sample N is
assumed to be distributed equally across all groups.

Format:  achisquare(f_obs,f_exp=None)
Returns: x2-value\n"""

    k = len(f_obs)
    if f_exp == None:
	f_exp = N.array([sum(f_obs)/float(k)] * len(f_obs),'f')
    f_exp = f_exp.astype('f')
    chisq = N.add.reduce((f_obs-f_exp)**2 / f_exp)
    return chisq


 def aks_2samp (data1,data2):
    """\nComputes the Kolmogorov-Smirnof statistic on 2 samples.  From
Numerical Recipies in C, page 493.  Returns KS D-value, prob.

Format:  aks_2samp(data1,data2)\n"""

    def probks(alam):
	fac = 2.0
	sum = 0.0
	termbf = 0.0
	a2 = -2.0*alam*alam
	for j in range(1,201):
	    term = fac*math.exp(a2*j*j)
	    sum = sum + term
	    if math.fabs(term) <= (0.001*termbf) or math.fabs(term) < (1.0e-8*sum):
		return sum
	    fac = -fac
	    termbf = math.fabs(term)
	return 1.0             # Get here only if fails to converge; was 0.0!!

    j1 = 0
    j2 = 0
    fn1 = 0.0
    fn2 = 0.0
    n1 = len(data1)
    n2 = len(data2)
    en1 = n1
    en2 = n2
    d = 0.0
    data1 = Numeric.sort(data1)
    data2 = Numeric.sort(data2)
    while j1 < n1 and j2 < n2:
	d1=data1[j1]
	d2=data2[j2]
	if d1 <= d2:
	    fn1 = (j1)/float(en1)
	    j1 = j1 + 1
	if d2 <= d1:
	    fn2 = (j2)/float(en2)
	    j2 = j2 + 1
	dt = math.fabs(fn2-fn1)
	if dt > d:
	    d = dt
    prob = probks(math.sqrt(en1*en2/float(en1+en2))*d)
    return d, prob


 def achisquare2d(contable):
    """\nCalculates a two-way chi square for n samples of scores and returns
the result.  If no expected frequencies are given, the total N is assumed to
be equally distributed across all groups.  Adapted from Numerical Recipies,
p.498.

Format:  achisquare2d(contable)
Returns: chisquare-statistic, df, probability, cramerV, contingency coeff\n"""

    TINY = 1.0e-30
    sum = 0.0
    ni = len(contable)
    nj = len(contable[0])
    nni = len(contable)
    nnj = len(contable[0])
    sumi = [0.0]*ni
    sumj = [0.0]*nj
    for i in range(ni):
	for j in range(nj):
	    sumi[i] = sumi[i] + contable[i][j]
	    sum = sum + contable[i][j]
	if sumi[i] == 0:
	    nni = nni - 1
    for j in range(nj):
	for i in range(ni):
	    sumj[j] = sumj[j] + contable[i][j]
	if sumj[j]==0:
	    nnj = nnj - 1
    df = nni*nnj-nni-nnj+1
    chisq = 0.0
    for i in range(ni):
	for j in range(nj):
	    expctd = sumj[j]*sumi[i] / float(sum)
	    temp = contable[i][j] - expctd
	    chisq = chisq + temp * temp / (expctd + TINY)
    prob = gammq(0.5*df,0.5*chisq)
    minij = min(nni,nnj) - 1
    cramerv = math.sqrt(chisq/(sum*minij))
    ccc = math.sqrt(chisq/(chisq+sum))
    return chisq, df, prob, cramerv, ccc


 def amannwhitneyu(x,y):
    """\nCalculates a Mann-Whitney U statistic on the provided scores and
returns the result.  Use only when the n in each condition is < 20 and you
have 2 independent samples of ranks.  REMEMBER: Mann-Whitney U is significant
if the u-obtained is LESS THAN or equal to the critical value of U.

Format:  amannwhitneyu(data)
Returns: u-statistic\n"""

    n1 = len(x)
    n2 = len(y)
    ranked = rankdata(x+y)
    rankx = ranked[0:n1]       # get the x-ranks
    ranky = ranked[n1:]        # the rest are y-ranks
    u1 = n1*n2 + (n1*(n1+1))/2.0 - sum(rankx)
    u2 = n1*n2 + (n2*(n2+1))/2.0 - sum(ranky)
    u = min(u1,u2)
    return u, 1.0 - zprob(u)


 def aranksums(x,y):
    """\nCalculates the rank sums statistic on the provided scores and
returns the result.  Use only when the n in each condition is > 20 and you
have 2 independent samples of ranks.  REMEMBER: Mann-Whitney U is significant
if the u-obtained is LESS THAN or equal to the critical value of U.

Format:  aranksums(x,y)
Returns: a z-statistic, crit val=1.96 (or 1.65 for 1-tailed)\n"""

    n1 = len(x)
    n2 = len(y)
    alldata = x+y
    ranked = rankdata(alldata)
    x = ranked[:n1]
    y = ranked[n1:]
    s = sum(x)
    expected = n1*(n1+n2+1) / 2.0
    z = (s - expected) / math.sqrt(n1*n2*(n1+n2+1)/12.0)
    return z, zprob(z)


 def awilcoxont(x,y):
    """\nCalculates the Wilcoxon T-test for related samples and returns the
result.  A non-parametric T-test.

Format:  awilcoxont(x,y)
Returns: a t-statistic, 1-tail probability estimate\n"""

    if len(x) <> len(y):
	raise ValueError, 'Unequal N in awilcoxont.  Aborting.'
    d=[]
    for i in range(len(x)):
	diff = x[i] - y[i]
	if diff <> 0:
	    d.append(diff)
    count = len(d)
    absd = map(abs,d)
    absranked = rankdata(absd)
    r_plus = 0
    r_minus = 0
    for i in range(len(absd)):
	if d[i] < 0:
	    r_minus = r_minus + absranked[i]
	else:
	    r_plus = r_plus + absranked[i]
    wt = min(r_plus, r_minus)
    mn = count * (count+1) * 0.25
    se =  math.sqrt(count*(count+1)*(2.0*count+1.0)/24.0)
    z = math.fabs(wt-mn) / se
    return wt, 1.0 - zprob(z)  # 1-tail prob. estimation!!!


 def akruskalwallish(*args):
    """\nThe Kruskal-Wallis H-test is a non-parametric ANOVA for 3 or more
groups, requiring at least 5 subjects in each group.  This function
calculates the Kruskal-Wallis H-test for 3 or more independent samples
and returns the result.  

Format:  akruskalwallish(*args)
Returns: H-statistic, probability\n"""

    args = list(args)
    n = [0]*len(args)
    all = []
    n = map(len,args)
    for i in range(len(args)):
	all = all + args[i]
    ranked = rankdata(all)
    for i in range(len(args)):
	args[i] = ranked[0:n[i]]
	del ranked[0:n[i]]
    rsums = []
    for i in range(len(args)):
	rsums.append(sum(args[i])**2)
	rsums[i] = rsums[i] / float(n[i])
    ssbn = sum(rsums)
    totaln = sum(n)
    h = 12.0 / (totaln*(totaln+1)) * ssbn - 3*(totaln+1)
    df = len(args) - 1
    return h, chisqprob(h,df)


 def afriedmanchisquare(*args):
    """\nFriedman Chi-Square is a non-parametric, one-way within-subjects
ANOVA.  This function calculates the Friedman Chi-square test for repeated
measures and returns the result, along with the associated probability
value.  It assumes 3 or more repeated measures.  Only 3 levels requires a
minimum of 10 subjects in the study.  Four levels requires 5 subjects per
level(??).

Format:  afriedmanchisquare(data)
Returns: chi-square statistic, probability\n"""

    args = list(args)
    k = len(args)
    if k < 3:
	raise ValueError, '\nLess than 3 levels.  Friedman test not appropriate.\n'
    n = len(args[0])
    args = map(rankdata,args)
    ssbn = 0
    for i in range(k):
	ssbn = ssbn + sum(args[i])**2
    chisq = 12.0 / (k*n*(k+1)) * ssbn - 3*n*(k+1)
    return chisq, chisqprob(chisq,k-1)


 def asigntest(x,y):
    """\nFUNCTION DOES NOT WORK.
Requires implementation of apobin, pobin, which requires a lot ..."""

    raise ValueError, "FUNCTION DOESN'T WORK YET"

    NSIGNTEST = 25          # According to Siegel
    if len(x) <> len(y):
	raise ValueError, 'Unequal n in asigntest.  Aborting.'
	return None
    r = N = 0
    for i in range(len(x)):
	if x[i] == y[i]:
	    pass
	else:
	    r = r + 1
	N = N + 1
    if r < N*0.5:
	r = N-r
    if N > NSIGNTEST:
	p = abinprob(N,1,2,r)    # approximation
    else:
	p = binprob(N,1,2,r)     # exact
    return


#####################################
####  APROBABILITY CALCULATIONS  ####
#####################################

 def achisqprob(chisq,df):
    """\nReturns the probability value associated with the provided chi-square
value and df.  Adapted from chisq.c in Gary Perlman's |Stat.

Format:  achisqprob(chisq,df)\n"""

    BIG = 20.0
    def ex(x):
	BIG = 20.0
	if x < -BIG:
	    return 0.0
	else:
	    return math.exp(x)

    if chisq <=0 or df < 1:
	return 1.0
    a = 0.5 * chisq
    if df%2 == 0:
	even = 1
    else:
	even = 0
    if df > 1:
	y = ex(-a)
    if even:
	s = y
    else:
	s = 2.0 * zprob(-math.sqrt(chisq))
    if (df > 2):
	chisq = 0.5 * (df - 1.0)
	if even:
	    z = 1.0
	else:
	    z = 0.5
	if a > BIG:
	    if even:
		e = 0.0
	    else:
		e = math.log(math.sqrt(math.pi))
	    c = math.log(a)
	    while (z <= chisq):
		e = math.log(z) + e
		s = s + ex(c*z-a-e)
		z = z + 1.0
	    return s
	else:
	    if even:
		e = 1.0
	    else:
		e = 1.0 / math.sqrt(math.pi) / math.sqrt(a)
		c = 0.0
		while (z <= chisq):
		    e = e * (a/float(z))
		    c = c + e
		    z = z + 1.0
		return (c*y+s)
    else:
	return s


 def aerfcc(x):
    """\nReturns the complementary error function erfc(x) with fractional
error everywhere less than 1.2e-7.  Adapted from Numerical Recipies.

Format:  aerfcc(x)\n"""

    z = abs(x)
    t = 1.0 / (1.0+0.5*z)
    ans = t * exp(-z*z-1.26551223 + t*(1.00002368+t*(0.37409196+t*(0.9678418+t*(-0.18628806+t*(0.27886807+t*(-1.13520398+t*(1.48851587+t*(-0.82215223+t*0.17087277)))))))))
    if x >= 0:
	return ans
    else:
	return 2.0 - ans


 def azprob(z):
    """\nReturns the probability value associated with the provided z-score.
Adapted from z.c in Gary Perlman's |Stat.

Format:  azprob(z)\n"""

    Z_MAX = 6.0    # maximum meaningful z-value
    if z == 0.0:
	x = 0.0
    else:
	y = 0.5 * math.fabs(z)
	if y >= (Z_MAX*0.5):
	    x = 1.0
	elif (y < 1.0):
	    w = y*y
	    x = ((((((((0.000124818987 * w
			-0.001075204047) * w +0.005198775019) * w
		      -0.019198292004) * w +0.059054035642) * w
		    -0.151968751364) * w +0.319152932694) * w
		  -0.531923007300) * w +0.797884560593) * y * 2.0
	else:
	    y = y - 2.0
	    x = (((((((((((((-0.000045255659 * y
			     +0.000152529290) * y -0.000019538132) * y
			   -0.000676904986) * y +0.001390604284) * y
			 -0.000794620820) * y -0.002034254874) * y
		       +0.006549791214) * y -0.010557625006) * y
		     +0.011630447319) * y -0.009279453341) * y
		   +0.005353579108) * y -0.002141268741) * y
		 +0.000535310849) * y +0.999936657524
    if z > 0.0:
	prob = ((x+1.0)*0.5)
    else:
	prob = ((1.0-x)*0.5)
    return prob


 def afprob (dfnum, dfden, F):
    """\nReturns the 1-tailed significance level (p-value) of an F
statistic given the degrees of freedom for the numerator (dfR-dfF) and
the degrees of freedom for the denominator (dfF).  It returns the string
'F is less than 1' when appropriate.

Format:  afprob(dfnum, dfden, F)   where usually dfnum=dfbn, dfden=dfwn\n"""

    p = betai(0.5*dfden, 0.5*dfnum, dfden/float(dfden+dfnum*F))
    return p


 def abetacf(a,b,x):
    """\nThis function evaluates the continued fraction form of the incomplete
Beta function, betai.  (Adapted from: Numerical Recipies in C.)

Format:  abetacf(a,b,x)\n"""

    ITMAX = 200
    EPS = 3.0e-7

    bm = az = am = 1.0
    qab = a+b
    qap = a+1.0
    qam = a-1.0
    bz = 1.0-qab*x/qap
    for i in range(ITMAX+1):
	em = float(i+1)
	tem = em + em
	d = em*(b-em)*x/((qam+tem)*(a+tem))
	ap = az + d*am
	bp = bz+d*bm
	d = -(a+em)*(qab+em)*x/((qap+tem)*(a+tem))
	app = ap+d*az
	bpp = bp+d*bz
	aold = az
	am = ap/bpp
	bm = bp/bpp
	az = app/bpp
	bz = 1.0
	if (abs(az-aold)<(EPS*abs(az))):
	    return az
    print 'a or b too big, or ITMAX too small in Betacf.'


 def agammln(xx):
    """Returns the gamma function of xx.
    Gamma(z) = Integral(0,infinity) of t^(z-1)exp(-t) dt.
(Adapted from: Numerical Recipies in C.)

Format:  agammln(xx)"""

    coeff = [76.18009173, -86.50532033, 24.01409822, -1.231739516,
	     0.120858003e-2, -0.536382e-5]
    x = xx - 1.0
    tmp = x + 5.5
    tmp = tmp - (x+0.5)*math.log(tmp)
    ser = 1.0
    for j in range(len(coeff)):
	x = x + 1
	ser = ser + coeff[j]/x
    return -tmp + math.log(2.50662827465*ser)


 def abetai(a,b,x):
    """\nReturns the incomplete beta function:

    I-sub-x(a,b) = 1/B(a,b)*(Integral(0,x) of t^(a-1)(1-t)^(b-1) dt)

where a,b>0 and B(a,b) = G(a)*G(b)/(G(a+b)) where G(a) is the gamma
function of a.  The continued fraction formulation is implemented here,
using the betacf function.  (Adapted from: Numerical Recipies in C.)

Format:  abetai(a,b,x)\n"""

    if (x<0.0 or x>1.0): print 'Bad x in Betai'
    if (x==0.0 or x==1.0):
	bt = 0.0
    else:
	bt = math.exp(gammln(a+b)-gammln(a)-gammln(b)+a*math.log(x)+b*
		      math.log(1.0-x))
    if (x<(a+1.0)/(a+b+2.0)):
	return bt*betacf(a,b,x)/float(a)
    else:
	return 1.0-bt*betacf(b,a,1.0-x)/float(b)


 def abinomial(n,p1,p2,r):

### ugh ... binomial.c in |Stat code

    raise ValueError, "FUNCTION DOESN'T WORK YET"

    maxpow = 0
    maxpow = max(maxpow,n)
    maxpow = max(maxpow,p2)
    return


 def abinprob(n,p1,p2,r):

    raise ValueError, "FUNCTION DOESN'T WORK YET"

    maxprob = (n*p1) / p2 + 1
    cum = 0.0
    for i in range(r,n+1):
	prob = binomial(n,p1,p2,i)
	if prob == BADPROB:
	    return BADPROB
	if prob == 0.0 and i > maxprob:
	    break
	cum = cum + prob
### ugh ... not done; binomial.c in |Stat code
    return


#####################################
#######  AANOVA CALCULATIONS  #######
#####################################

 def aF_oneway(*lists):
    """\nPerforms a 1-way ANOVA, returning an F-value and probability given
any number of groups.  From Heiman, pp.394-7.

Format:  aF_oneway (*lists)    where *lists is a set of lists, one per
                                  treatment group\n"""

    a = len(lists)                 # ANOVA on 'a' groups, each in it's own list
    means = [0]*a
    vars = [0]*a
    ns = [0]*a
    alldata = []
    tmp = map(N.array,lists)
    means = map(amean,tmp)
    vars = map(avar,tmp)
    ns = map(len,lists)
    for i in range(len(lists)):
	alldata = alldata + lists[i]
    alldata = N.array(alldata)
    bign = len(alldata)
    sstot = ass(alldata)-(asumsquared(alldata)/float(bign))
    ssbn = 0
    for list in lists:
	ssbn = ssbn + asumsquared(N.array(list))/float(len(list))
    ssbn = ssbn - (asumsquared(alldata)/float(bign))
    sswn = sstot-ssbn
    dfbn = a-1
    dfwn = bign - a
    msb = ssbn/float(dfbn)
    msw = sswn/float(dfwn)
    f = msb/msw
    prob = fprob(dfbn,dfwn,f)
    return f, prob


 def aF_nway_btw (data):
    """\nPerforms an n-way ANOVA, returning F-values and probability given
any number of BETWEEN-SUBJECTS factors.  From Heiman, pp.419-32.  Like
|Stat imput (one data point per row).  In the unequal-N case, it does
A-entered first, then B-entered first (Type I ss--Maxwell & Delaney 298-95).
DOESN'T WORK YET!!!!!!!!!!

Format:  aF_nway_btw(data)    where 'data' is a list of lists, row=1 RT\n"""

    raise ValueError, "FUNCTION DOESN'T WORK YET"

    print 'F_nway_btw currently uses Type I sums-of-squares.  Type III is better for       non-orthogonal designs.'
    ns = [0]*len(data[0])
    alluniqueslist = [0]*len(data[0])
    for column in range(len(data[0])):   # get Ns and unique vals for ALL cols
	alluniqueslist[column] = pstat.unique(pstat.colex(data,column))
	ns[column] = len(alluniqueslist[column])
    numfactors = len(data[0][1:-1])
    uniquelevels = alluniqueslist[1:-1]  # unique levels in columns 1 to n-1

### CALC SS FOR EACH MAIN EFFECT
    means = [0]*numfactors        # "factors" number of slots for means
    n = [0]*numfactors            # "factors" number of slots for Ns
    sumsqd = [0]*numfactors       # "factors" number of slots for SS
    for factor in range(numfactors):
	numlevels = len(uniquelevels[factor])
	means[factor] = [0]*numlevels  # 1 mean/factor-level
	n[factor] = [0]*numlevels      # 1 N/factor-level
	for level in range(numlevels):
            leveltoget = uniquelevels[factor][level]
	    tmprts = pstat.colex(pstat.linexand(data,factor+1,leveltoget),
				 len(data[0])-1)       # extract block RTs
	    means[factor][level] = mean(tmprts)        # store block mean
	    n[factor][level] = len(tmprts)             # store block N, & SS
	    sumsqd[factor] = sumsqd[factor] + sumsquared(tmprts)/float(len(tmprts))
    allrts = pstat.colex(data,len(data[0])-1)          # get all RTs, SS
    bign = len(allrts)
    sstot = ss(allrts) - sumsquared(allrts)/float(bign)
    ssx = [0]*numfactors                               # calc SSx (x=[A,B,...])
    ssx = N.array(sumsqd) - sumsquared(allrts)/float(bign)
    print

### CALC INTERACTION SSs

    ### THIS FUNCTION DOESN'T WORK
    def interactss(data,prevmeans,interacttype):
	"""interacttype = 2 (for 2-way interacts), 3 for 3-way, etc."""
	numinteractions = N.add.reduce(len(data[1:-1])-interacttype+1)
	for i in range(numinteractions):
	    mns = 0
	    ssi = 0
	return mns, ssi

    ### NONE OF THIS WORKS
    intss = [0]*(numlevels-1)
    mns = []
    for itype in range(2,numlevels+1):
	numinteracts = N.add.reduce(N.arange(numlevels-itype+1)+1)
#	newmns, ssint[itype-2] = interactss(data,itype)
#	mns.append(newmns)

    ### FROM HERE ON IT WORKS FINE (I.E., INTERACTIONS AREN'T IMPLEMENTED YET)
    factordata = pstat.colex(data,range(1,numfactors+1))
    uniquecells = pstat.unique(factordata)        # get unique factor/combos
    cellsqd = [0]*len(uniquecells)
    for cell in range(len(uniquecells)):
	factrts = pstat.abut(factordata,pstat.colex(data,len(data[0])-1))
	tmpdata = pstat.linexand(factrts,range(len(factordata[0])),
				 uniquecells[cell])
	tmprts = pstat.colex(tmpdata,len(tmpdata[0])-1)
#	print "Factor = ", cell, uniquecells[cell], tmprts
	cellsqd[cell] = sumsquared(tmprts)/float(len(tmprts))
    print cellsqd
    ssbn = N.add.reduce(N.array(cellsqd))
    ssbn = ssbn - sumsquared(allrts)/float(bign)
    sswn = sstot - ssbn
    ssint = ssbn - (sum(ssx))

    df = [0]*len(ssx)
    for factor in range(len(ssx)):
	df[factor] = len(uniquelevels[factor])-1
    dfbn = N.multiply.reduce(N.array(df))
    cells = len(uniquecells)
    dfwn = len(data) - cells
    dftot = len(data) - 1
    dfint = N.multiply.reduce(N.array(df))

    print ssx, df, ssint, dfint
    ms = list(N.array(ssx)/N.array(df),'f')
    mswn = sswn / float(dfwn)
    msbn = ssbn / float(dfbn)
    msint = ssint / float(dfint)
    print
    print msint
    print
    ms.append(msint)
    print ms

    print
    print ssx, ssbn, sswn, sstot, df, dfbn, dfwn,dftot, mswn, msbn
    print
    fs = N.array(ms)/mswn
    for f in fs:
	prob = fprob(dfbn,dfwn,f)
	print f, round(prob,5)
    return


 def asstot(Y):
    """\nReturns the total sums of squares of a 2D matrix (subtract
MATRIX mean from each element, square the differences and sum across
all matrix elements).

Format:  asstot(Y)    where Y is a 2D array\n"""

    if type(Y) <> N.ArrayType:
	Y = N.array(Y)
    subjpergroup = Y.shape[0]
    try:
	numgroups = Y.shape[1]
    except IndexError:
	print "Only a 1D matrix."
	numgroups = 1
    groupmeans = N.add.reduce(Y,1)/float(subjpergroup)            # Yjbar
    matrixmean = N.add.reduce(groupmeans,0)/float(numgroups)      # Ybar

    Rerror = Y - matrixmean
    return ss(Rerror)


 def assbn(Y):
    """\nReturns the between-groups sums of squares from a 2D matrix.
Subjects are contained in rows, conditions are separate columns.
Empty cells counted as zero.

Format:  assbn(Y)    where Y is a 2D array\n"""

    if type(Y) <> N.ArrayType:
	Y = N.array(Y)
    subjpergroup = Y.shape[0]
    numgroups = Y.shape[1]
    groupmeans = N.add.reduce(Y,0)/float(subjpergroup)            # Yjbar
    matrixmean = N.add.reduce(groupmeans,0)/float(numgroups)      # Ybar
                    # ^^^ not good for generic mean (must be using equal-N)

    delta = groupmeans - matrixmean
    return subjpergroup*ss(delta)


 def amsbn(Y):
    """\nReturns the within-groups mean sum of squares of a 2D matrix.  Rows
contain individual subjects, columns contain scores for each condition.

Format:  amsbn(Y)   where Y is a 2D array\n"""

    if (type(Y) <> N.ArrayType):
	Y = N.array(Y)
    subjpergroup = Y.shape[0]        # number of subjects (rows)
    try:
	numgroups = Y.shape[1]
    except IndexError:
	print "Only a 1D matrix."
	numgroups = 1
    groupmeans = N.add.reduce(Y,0)/float(subjpergroup)  # Yjbar
    df = numgroups - 1                                        # i.e., a-1
    return ssbn(Y)/float(df)


 def asswn(Y):
    """\nReturns the within-groups sum of squares of a 2D matrix.  Rows
contain individual subjects, columns contain scores for each condition.

Format:  asswn(Y)    where Y is a 2D array\n"""

    if (type(Y) <> N.ArrayType):
	Y = N.array(Y)
    subjpergroup = Y.shape[0]
    try:
	numgroups = Y.shape[1]
    except IndexError:
	print "Only a 1D matrix."
	numgroups = 1
    groupmeans = N.add.reduce(Y,0)/float(subjpergroup)          # Yjbar
    Ferror = N.subtract(Y,groupmeans)
    return ss(Ferror)


 def amswn(Y):
    """\nReturns the mean sum of squares within groups of a 2D matrix.  Rows
contain individual subjects, columns contain scores for each condition.

Format:  amswn(Y)    where Y is a 2D array\n"""

    if (type(Y) <> N.ArrayType):
	Y = N.array(Y)
    subjpergroup = Y.shape[0]        # number of subjects (rows)
    try:
	numgroups = Y.shape[1]
    except IndexError:
	print "Only a 1D matrix."
	numgroups = 1
    groupmeans = N.add.reduce(Y,0)/float(subjpergroup)            # Yjbar
    df = (subjpergroup*numgroups) - (numgroups)        # i.e., N-a
    return sswn(Y)/float(df)


 def asslin(X,Y):
    """\nReturns the sums of squares for linear contrast from a 2D matrix.
Requires a 1D list of x-values, and a 2D-list of y-values (one column
for each x-value, one row per subject).

Format:  asslin(X,Y)    where X and Y are 2D arrays\n"""

    if type(X) <> N.ArrayType:
	X = N.array(X)
    if type(Y) <> N.ArrayType:
	Y = N.array(Y)
    subjpergroup = Y.shape[0]
    numgroups = Y.shape[1]
    groupmeans = N.add.reduce(Y,0)/float(subjpergroup)           # Yjbar

    Xbar = N.add.reduce(X,0) / float(numgroups)
    Xerror = X - N.array(Xbar)			# array of Xj-Xbar = cj
    psi = 0
    for j in range(len(Xerror)):
        psi = psi + Xerror[j]*groupmeans[j]
    return psi*psi / (ss(Xerror)/float(subjpergroup))


 def alin_trend(X,Y):
    """\nReturns the F-statistic for a linear trend analysis (linear trend
vs. no trend).  Requires a 1D list of x-values, and a 2D-list of y-values
(one column for each x-value, one row per subject).

Format:  alin_trend(X,Y)    where X=1D list of x-values, Y=2D list of y-vals\n"""

    if type(X) <> N.ArrayType:
	X = N.array(X)
    if type(Y) <> N.ArrayType:
	Y = N.array(Y)
    return sslin(X,Y)/mswn(Y)


 def aF_multivar (data,scorespersubj):
    """\nDoes multivariate ANOVAs.

Format:  aF_multivar(data,scorespersubj)    data = |Stat type,
                scorespersubj = how many cols of dependent measures\n"""
    return


 def aF_nway_wi (data):
    """\nDoes within-subject ANOVA designs.   DOESN'T WORK YET!

Format:  aF_nway_wi(data)    data = |Stat format\n"""

    raise ValueError, "FUNCTION DOESN'T WORK YET"

    ns = [0]*len(data[0])
    alluniqueslist = [0]*len(data[0])
    for column in range(len(data[0])):
	alluniqueslist[column] = pstat.unique(pstat.colex(data,column))
	ns[column] = len(alluniqueslist[column])
    numfactors = len(data[0][1:-1])
    uniquelevels = alluniqueslist[1:-1]  # unique levels in columns 1 to n-1

### CALC D-VARIABLES (Maxwell & Delaney pp.622-4)
    coefflist = ((),
		 ((-1,1)),
		 ((-1,0,1),(1,-2,1)),
		 ((-3,-1,1,3),(1,-1,-1,1),(-1,3,-3,1)),
		 ((-2,-1,0,1,2),(2,-1,-2,-1,-2),(-1,2,0,-2,1),(1,-4,6,-4,1)),
		 ((-5,-3,-1,1,3,5),(5,-1,-4,-4,-1,5),(-5,7,4,-4,-7,5),
		  (1,-3,2,2,-3,1),(-1,5,-10,10,-5,1)))
    withinsubj = findwithin(data)
    for factor in range(len(uniquelevels)):
	if withinsubj[factor] == 1:
	    numlevels = len(uniquelevels[factor])    # get num levels of factor
	    coeff = N.array(coeff[numlevels-1])      # get coeff for D-vars
	    tmpdata = [0]*numlevels                  # prepare for data
	    for level in range(numlevels):           # get all data for factor
		rows = linexor(data,factor+1,uniquelevels[factor][level])
		tmpdata[level] = colex(rows,len(rows[0])-1)
	    tmpdata = N.array(tmpdata,'f')
	    newdata = [0]*numlevels-1
	    for level in range(len(coeff)):          # calc D-variables
		newdata[level] = N.add.reduce(tmpdata*coeff[level],1)

    numvariables = numlevels-1
    F_multivar(newdata,numvariables)
    return


 def aF_test (data):
    """\nDetermines whether data requires a between or within subjects anova,
and then runs the appropriate test.  General form for F_nway_btw and
F_multivar.

Format:  aF_test(data)     where data is 1 datum per row.\n"""

    subjects = pstat.unique(pstat.colex(data,0))
    if len(subjects) == len(data):      # fully between if #subj=#datapoints
	F_nway_btw(data)
    else:
	F_nway_wi(data)
    return


 def aF_value (ER,EF,dfR,dfF):
    """Returns an F-statistic given the following:
	ER  = error associated with the null hypothesis (the Restricted model)
	EF  = error associated with the alternate hypothesis (the Full model)
	dfR = degrees of freedom the Restricted model
	dfF = degrees of freedom associated with the Restricted model"""

    return ((ER-EF)/float(dfR-dfF) / (EF/float(dfF)))


 def asstot(Y):
    """\nReturns the total sums of squares of a 2D matrix (subtract
MATRIX mean from each element, square the differences and sum across
all matrix elements).

Format:  asstot(Y)    where Y is a 2D array\n"""

    if type(Y) <> N.ArrayType:
	Y = N.array(Y)
    subjpergroup = Y.shape[0]
    try:
	numgroups = Y.shape[1]
    except IndexError:
	print "Only a 1D matrix."
	numgroups = 1
    groupmeans = N.add.reduce(Y,1)/float(subjpergroup)            # Yjbar
    matrixmean = N.add.reduce(groupmeans,0)/float(numgroups)      # Ybar

    Rerror = Y - matrixmean
    return ss(Rerror)


 def assbn(Y):
    """\nReturns the between-groups sums of squares from a 2D matrix.
Subjects are contained in rows, conditions are separate columns.
Empty cells counted as zero.

Format:  assbn(Y)    where Y is a 2D array\n"""

    if type(Y) <> N.ArrayType:
	Y = N.array(Y)
    subjpergroup = Y.shape[0]
    numgroups = Y.shape[1]
    groupmeans = N.add.reduce(Y,0)/float(subjpergroup)            # Yjbar
    matrixmean = N.add.reduce(groupmeans,0)/float(numgroups)      # Ybar
                    # ^^^ not good for generic mean (must be using equal-N)

    delta = groupmeans - matrixmean
    return subjpergroup*ss(delta)


 def amsbn(Y):
    """\nReturns the within-groups mean sum of squares of a 2D matrix.  Rows
contain individual subjects, columns contain scores for each condition.

Format:  amsbn(Y)   where Y is a 2D array\n"""

    if (type(Y) <> N.ArrayType):
	Y = N.array(Y)
    subjpergroup = Y.shape[0]        # number of subjects (rows)
    try:
	numgroups = Y.shape[1]
    except IndexError:
	print "Only a 1D matrix."
	numgroups = 1
    groupmeans = N.add.reduce(Y,0)/float(subjpergroup)  # Yjbar
    df = numgroups - 1                                        # i.e., a-1
    return ssbn(Y)/float(df)


 def asswn(Y):
    """\nReturns the within-groups sum of squares of a 2D matrix.  Rows
contain individual subjects, columns contain scores for each condition.

Format:  asswn(Y)    where Y is a 2D array\n"""

    if (type(Y) <> N.ArrayType):
	Y = N.array(Y)
    subjpergroup = Y.shape[0]
    try:
	numgroups = Y.shape[1]
    except IndexError:
	print "Only a 1D matrix."
	numgroups = 1
    groupmeans = N.add.reduce(Y,0)/float(subjpergroup)          # Yjbar
    Ferror = N.subtract(Y,groupmeans)
    return ss(Ferror)


 def amswn(Y):
    """\nReturns the mean sum of squares within groups of a 2D matrix.  Rows
contain individual subjects, columns contain scores for each condition.

Format:  amswn(Y)    where Y is a 2D array\n"""

    if (type(Y) <> N.ArrayType):
	Y = N.array(Y)
    subjpergroup = Y.shape[0]        # number of subjects (rows)
    try:
	numgroups = Y.shape[1]
    except IndexError:
	print "Only a 1D matrix."
	numgroups = 1
    groupmeans = N.add.reduce(Y,0)/float(subjpergroup)            # Yjbar
    df = (subjpergroup*numgroups) - (numgroups)        # i.e., N-a
    return sswn(Y)/float(df)


 def asslin(X,Y):
    """\nReturns the sums of squares for linear contrast from a 2D matrix.
Requires a 1D list of x-values, and a 2D-list of y-values (one column
for each x-value, one row per subject).

Format:  asslin(X,Y)    where X and Y are 2D arrays\n"""

    if type(X) <> N.ArrayType:
	X = N.array(X)
    if type(Y) <> N.ArrayType:
	Y = N.array(Y)
    subjpergroup = Y.shape[0]
    numgroups = Y.shape[1]
    groupmeans = N.add.reduce(Y,0)/float(subjpergroup)           # Yjbar

    Xbar = N.add.reduce(X,0) / float(numgroups)
    Xerror = X - N.array(Xbar)			# array of Xj-Xbar = cj
    psi = 0
    for j in range(len(Xerror)):
        psi = psi + Xerror[j]*groupmeans[j]
    return psi*psi / (ss(Xerror)/float(subjpergroup))


 def alin_trend(X,Y):
    """\nReturns the F-statistic for a linear trend analysis (linear trend
vs. no trend).  Requires a 1D list of x-values, and a 2D-list of y-values
(one column for each x-value, one row per subject).

Format:  alin_trend(X,Y)    where X=1D list of x-values, Y=2D list of y-vals\n"""

    if type(X) <> N.ArrayType:
	X = N.array(X)
    if type(Y) <> N.ArrayType:
	Y = N.array(Y)
    return sslin(X,Y)/mswn(Y)


 def ainvsquare(num):
        return 1/float(num*num)



 def member(factor,source):
     return (1 << factor) & source != 0

 def setsize(source):
     size = 0
     for bit in source:
	 if bit == 1:
	     size = size + 1
     return size

 def subset (a,b):
     return (a&b)==a

 def numlevels(source,Nlevels):
     for i in range(30):
	 if 1<<i >= source:
	     break
     levelcount = 1
     for j in range(i):
	 if subset(1<<j,source):
	     levelcount = levelcount * Nlevels[j] - 1
     return levelcount

 def makelist(source,ncols):
     levellist = []
     for j in range(ncols):
	 if subset(1<<j,source):
	     levellist.append(j)
     return levellist


 def F_multivariate (data):
    """\nDoes between- and within-subject ANOVA designs.   DOESN'T WORK YET!

Format:  F_multivariate(data)    data = |Stat format\n"""

    raise ValueError, "FUNCTION F_multivariate() DOESN'T WORK YET."

    alluniqueslist = [0]*(len(data[0])-1)
    Nlevels = [0]*(len(data[0])-1)
    for column in range(len(Nlevels)):
	alluniqueslist[column] = pstat.unique(pstat.colex(data,column))
	Nlevels[column] = len(alluniqueslist[column])
    Ncells = N.multiply.reduce(Nlevels[1:])
    Nfactors = len(Nlevels[1:])
    Nsubjects = len(alluniqueslist[0])

    Bwithins = findwithin(data)
    Wcolumns = makelist(Bwithins,Nfactors+1)  # get list of cols of w/i factors
    Nwithin = len(Wcolumns)
    Nwlevels = [0] * (Nwithin+1)
    Wscols = [0] + Wcolumns
    for column in range(Nwithin+1):
	Nwlevels[column] = len(alluniqueslist[Wscols[column]])
    print Wscols

    Nallsources = 2**Nfactors
    Nwsources = 2**Nwithin - 1       # num within-subject level-combos
    Nbsources = Nallsources - Nwsources

    print '\n# Levels  =',Nlevels
    print '# Cells   =',Ncells
    print '# Factors =',Nfactors
    print '# Subjs   =', Nsubjects
    print '# Wfact   =',Nwithin
    print 'Within subject factor columns = ',Wcolumns
    print

    if Nwithin == 0:
	# DO BETWEEN-SUBJECTS DATA ANALYSIS
	raise ValueError('Between subjects analysis not implemented yet.')

    else:  # Nwithin > 0
### CALC M-VARIABLE
	Bscols = makelist(~Bwithins,Nfactors+1)
	print 'Bscols = ',Bscols
	M = pstat.collapse(data,Bscols,-1)
	print 'M-variable = '
	pstat.pl(M)

### CALC D-VARIABLES (Maxwell & Delaney pp.622-4)
	coefflist = [[[1]],
		     [[-1,1]],
		     [[-1,0,1],[1,-2,1]],
		     [[-3,-1,1,3],[1,-1,-1,1],[-1,3,-3,1]],
		     [[-2,-1,0,1,2],[2,-1,-2,-1,-2],[-1,2,0,-2,1],[1,-4,6,-4,1]],
		     [[-5,-3,-1,1,3,5],[5,-1,-4,-4,-1,5],[-5,7,4,-4,-7,5],
		      [1,-3,2,2,-3,1],[-1,5,-10,10,-5,1]]]

	dindex = 0
	NDs = [0]* Nwsources            # GET NUM D-LEVELS PER LEVEL-COMBO
	for source in range(Nwsources):
	    if subset(source,Bwithins):
		NDs[dindex] = numlevels(source,Nlevels)
		dindex = dindex + 1

	DARRAY = N.zeros(Nlevels,'f')   # CREATE DARRAY: EACH FACTOR IN OWN DIM
	counts = N.zeros(Nlevels,'f')
	print DARRAY.shape
	for i in range(len(data)): # for every datapoint
	    idx = []
	    for j in range(Nfactors+1): # get n-D bin idx for this datapoint
		new = alluniqueslist[j].index(data[i][j])
		idx.append(new)
	    DARRAY[idx] = DARRAY[idx] + data[i][-1]
	    counts[idx] = counts[idx] + 1
	DARRAY = DARRAY / N.clip(counts,1,max(N.ravel(counts))) # calc means
	print DARRAY.shape, min(DARRAY.flat), max(DARRAY.flat)
	print counts.shape, min(counts.flat), max(counts.flat)

	DWARRAY = N.zeros(Nwlevels,'f') #CREATE DWARRAY: EACH FACTOR IN OWN DIM
	wcounts = N.zeros(Nwlevels,'f')
	print DWARRAY.shape
	for i in range(len(data)): # for every datapoint
	    idx = []
	    for j in range(Nfactors+1): # get n-D bin idx for this datapoint
		new = alluniqueslist[j].index(data[i][j])
		idx.append(new)
	    idx = N.take(N.array(idx),Wscols)
	    DWARRAY[idx] = DWARRAY[idx] + data[i][-1]
	    wcounts[idx] = wcounts[idx] + 1
	DWARRAY = DWARRAY / N.clip(wcounts,1,max(N.ravel(wcounts))) #calc means
	print DWARRAY.shape, min(DWARRAY.flat), max(DWARRAY.flat)
	print wcounts.shape, min(wcounts.flat), max(wcounts.flat)

### AVG OVER NON-SOURCES (BUT NOT SUBJS)
	print 'Binary within-subjects: ',Bwithins
	dcount = -1
	Wsources = []
	D = N.zeros(Nwsources,N.PyObject)  # one slot for each Dx, 2**Nwithin
	for source in range(3,Nallsources,2):  # sources that incl. subjects
	    dcount = dcount + 1
	    print '\nSOURCE = ',source
	    if 1:  #subset(source,Bwithins+1):      # and are within-only terms
		dwsc = DWARRAY
		Bnonsource = (Nwsources*2+1) & ~source
		Wnonsourcecolumns = makelist(Bnonsource,Nfactors+1) # get source w/i cols
		print source,makelist(source,Nfactors+1)
		print Bnonsource,makelist(Bnonsource,Nfactors+1)
		for i in range(len(Wnonsourcecolumns)):
		    dwsc = amean(dwsc,Wnonsourcecolumns[i])  #collapse all nonsource columns
		mns = dwsc
		print dwsc.shape, mns.shape, DWARRAY.shape
		print source, Wnonsourcecolumns, 'MNS.shape = ',mns.shape

### NOW, ACTUALLY COMPUTE THE D-VARIABLE ENTRIES FROM MNS

### CREATE LIST OF COEFF-COMBINATIONS TO DO (len=e-1, f-1, (e-1)*(f-1), etc...)
		dvarshape = N.array(mns.shape[1:]) - 1
		idxarray = N.indices(dvarshape)
		newshape = [idxarray.shape[0],N.multiply.reduce(idxarray.shape[1:])]
		print idxarray, idxarray.shape, newshape
		indxlist = N.swapaxes(N.reshape(idxarray,newshape),0,1)
		print indxlist.shape

### CREATE ALL D-VARIABLES FOR THIS COMBINATION OF FACTORS
		for i in range(len(indxlist)):

### FILL UP COEFFMATRIX (OF SHAPE = MNS) WITH CORRECT COEFFS FOR 1 D-VAR
		    coeffmatrix = N.ones(mns.shape)
		    for wfactor in range(len(coeffmatrix.shape[1:])):
			coeffmatrix = N.swapaxes(coeffmatrix,0,wfactor+1)#swap it up
			nlevels = coeffmatrix.shape[0]
			print nlevels, i, wfactor, indxlist, coeffmatrix.shape
			nextcoeff = coefflist[nlevels-1][indxlist[i,wfactor]]
			print coefflist[nlevels-1], 'Nextcoeff=',nextcoeff
			for j in range(nlevels):
			    coeffmatrix[j] = coeffmatrix[j] * nextcoeff[j]
#			    print nextcoeff[j], type(nextcoeff[j]), coeffmatrix[j]
			coeffmatrix = N.swapaxes(coeffmatrix,0,wfactor+1)#put it back
### CALCULATE D VARIABLE
		    print 'MNS = ',mns
		    print 'coef= ',coeffmatrix
		    scratch = coeffmatrix * mns
		    for j in range(len(coeffmatrix.shape[1:])):
			scratch = N.add.reduce(scratch,1) # unweighted means(?)
		    print scratch.shape
		    if D[dcount] == 0:
			D[dcount] = scratch
		    else:
			D[dcount] = pstat.aabut(D[dcount],scratch)
		Wsources.append(source)
		print '\nD = \n',D, D[0].shape
		print len(D)



### DONE CREATING M AND D VARIABLES ... TIME FOR SOME SS WORK

	print M
	for a in D:
	    print a
	    print Ef(a)
	    print Er(a)
	    print LA.determinant(Ef(a)), LA.determinant(Er(a))
	for source in range(2,Nallsources,2):  # calc F for sources w/o subj
	    print source, ~Bwithins+1
	    if not (source & Bwithins):  # btw-subjects vars only?
###
### BETWEEN-SUBJECTS VARIABLES ONLY, use M variable for analysis
###
### FIGURE OUT WHICH VARS TO RESTRICT, see p.680 (Maxwell&Delaney)
		sourcecols = makelist(source,Nfactors+1)
		M = pstat.collapse(data,sourcecols,-1)
		
### CALCULATE Er, Ef ... see pp. 282-3 (Maxwell & Delaney)
		er = 1000
		ef = 100


### CALCULATE dfnum, dfden
		sourceNs = pstat.colex([Nlevels],makelist(source,Nfactors+1))
		dfnum = N.multiply.reduce(N.array(sourceNs)-1)
		BNs = pstat.colex([Nlevels],makelist(-~Bwithins,Nfactors+1))
		dfden = Nsubjects - N.multiply.reduce(BNs[0][1:])
		print BNs, sourceNs, dfnum, dfden, Er(a), Ef(a)

###
### SOME WITHIN-SUBJECTS VARIABLES INCLUDED
###
	    else:  # source has within-subjects variables
### FIGURE OUT WHICH D-VAR TO USE
		

### FIGURE OUT WHICH VARS TO RESTRICT, see p.680 (Maxwell&Delaney)
		
### CALCULATE Er, Ef
		er = 1000
		ef = 100

### CALCULATE dfnum, dfden
		sourceNs = pstat.colex([Nlevels],makelist(source,Nfactors+1))
		dfnum = N.multiply.reduce(N.ravel(N.array(sourceNs))-1)
		if subset(source,Bwithins):
		    WNs = pstat.colex([Nlevels],makelist(Bwithins,Nfactors+1))
		    dfden = Nsubjects-N.multiply.reduce(WNs[0][1:]) - dfnum+1
		else:
		    ### use Rao's approximation, p.680 Maxwell & Delaney
		    dh = dfnum  # no. of indep restricted params (per dep var)
		    p = 1       # no. of dependent variables
		    m = Nsubjects - N.multiply.reduce(BNs[0][1:]) +dh -0.5*(p+dh+1)
		    d_en = float(p**2 + dh**2-5)
		    if d_en == 0.0:
			d_en = 1.0
		    q = math.sqrt(((p*dh)**2-4) / d_en)
		    dfden = m*q - 0.5*p*dh + 1

### OUTPUT RESULTS for either WI or BTW
	    f = F_value_multivariate(er, ef, dfnum, dfden)
	    prob = fprob(dfnum, dfden, f)
	    outputfstats(er-ef, ef, dfnum, dfden, f, prob)
    print Nallsources,Wsources
    return


 def outputfstats(Enum, Eden, dfnum, dfden, f, prob):
     Enum = round(Enum,3)
     Eden = round(Eden,3)
     dfnum = round(Enum,3)
     dfden = round(dfden,3)
     f = round(f,3)
     prob = round(prob,3)
     suffix = ''                       # for *s after the p-value
     if  prob < 0.001:  suffix = '  ***'
     elif prob < 0.01:  suffix = '  **'
     elif prob < 0.05:  suffix = '  *'
     title = [['EF/ER','DF','Mean Square','F-value','prob','']]
     lofl = title+[[Enum, dfnum, round(Enum/float(dfnum),3), f, prob, suffix],
		   [Eden, dfden, round(Eden/float(dfden),3),'','','']]
     pstat.printcc(lofl)
     return


 def F_value_multivariate(ER, EF, dfnum, dfden):
     if type(ER) in [IntType, FloatType]:
	 ER = N.array([[ER]])
     if type(EF) in [IntType, FloatType]:
	 EF = N.array([[EF]])
     print dfnum
     n_um = (LA.determinant(ER) - LA.determinant(EF)) / float(dfnum)
     d_en = LA.determinant(EF) / float(dfden)
     return n_um / d_en



 def Ef(a):
     """Calculates error for the full model on array a.  Assumes subjects
is first dimension of array, and remaining dimensions include ALL data
relevant for the full-model calculations.

Format:  Ef(a)
Returns: array for multivariate F calculation, raw D-score minus group means.\n"""

     if len(a.shape)==1:
	 a = a[:,N.NewAxis]
     errors = a - (N.add.reduce(a,0) / float(len(a))) # subtract group mns
     levels = a.shape[-1]
     ef = N.zeros((levels,levels),'f')
     errors = N.swapaxes(errors,0,1)
     for i in range(levels):
	 for j in range(i,levels):
	     ef[i,j] = N.add.reduce(errors[i]*errors[j])
	     ef[j,i] = N.add.reduce(errors[i]*errors[j])
     return ef


 def Er(a):
     """\nAssumes subjects is first dimension of array a.

Format:  Er(a)
Returns: array for multivariate F calculation, based on raw D-scores.\n"""

     if len(a.shape)==1:
	 a = a[:,N.NewAxis]
     errors = a
     levels = a.shape[-1]
     er = N.zeros((levels,levels),'f')
     errors = N.swapaxes(errors,0,1)
     for i in range(levels):
	 for j in range(i,levels):
	     er[i,j] = N.add.reduce(errors[i]*errors[j])
	     er[j,i] = N.add.reduce(errors[i]*errors[j])
     return er


 def dcollapse (listoflists,keepcols,collapsecols,sterr=None,ns=None):
    """\nAverages data in collapsecol, keeping all unique items in keepcols (using
unique, which keeps unique LISTS of column numbers), retaining the unique
sets of values in keepcols, the mean for each.  Setting a value for sterr
and/or ns will retain the sterr and N of the calculated means.

Format:  collapse (listoflists,keepcols,collapsecols,sterr=None,ns=None)
Returns: a list of lists with all unique permutations of entries appearing in
         columns ("conditions") specified by keepcols, abutted with the mean
         of each column specified by collapsecols.\n"""

    if type(keepcols) not in [ListType,TupleType]:
	keepcols = [keepcols]
    if type(collapsecols) not in [ListType,TupleType]:
	collapsecols = [collapsecols]
    if keepcols == []:
	means = [0]*len(collapsecols)
	for i in range(len(collapsecols)):
	    avgcol = pstat.colex(listoflists,collapsecols[i])
	    means[i] = mean(avgcol)
	return means
    else:
	values = pstat.colex(listoflists,keepcols)
	uniques = pstat.unique(values)
	uniques.sort()
	newlist = []
	if type(keepcols) not in [ListType,TupleType]:  keepcols = [keepcols]
	for item in uniques:
	    if type(item) not in [ListType,TupleType]:  item =[item]
	    tmprows = pstat.linexand(listoflists,keepcols,item)
	    for col in collapsecols:
		avgcol = pstat.colex(tmprows,col)
		item.append(mean(avgcol))
		if sterr:
		    if len(avgcol)>1:
			item.append(stats.sterr(avgcol))
		    else:
			item.append('N/A')
		if ns:
		    item.append(len(avgcol))
		newlist.append(item)
	return newlist


#####################################
#######  ASUPPORT FUNCTIONS  ########
#####################################

 def ass(inarray, dimension=-1):
    """\nSquares each value in the passed array, adds these squares & returns
the result.  Unfortunate function name. :-)  Defaults to ALL values in
the array.  Use dimension=-1 to flatten array first.

Format:  ass(inarray,dimension=-1)\n"""

    if dimension == -1:
	inarray = N.ravel(inarray)
	dimension = 0
    return N.add.reduce(inarray*inarray,dimension)


 def asumsquared(inarray, dimension=-1):
    """\nAdds the values in the passed array, squares that sum, and returns
the result.  Defaults to ALL values in the array.  Use -1 to flatten array
first.

Format:  asumsquared(inarray,dimension=-1)\n"""

    if dimension == -1:
	inarray = N.ravel(inarray)
	dimension = 0
    sum = N.add.reduce(inarray,dimension)
    return sum*sum


 def ashellsort(inlist):
    """\nShellsort algorithm.  Sorts a 1D-list.

Format:  ashellsort(inlist)
Returns: sorted-inlist, sorting-index-vector (for original list)\n"""

    n = len(inlist)
    svec = copy.deepcopy(inlist)
    ivec = range(n)
    gap = n/2   # integer division needed
    while gap >0:
	for i in range(gap,n):
	    for j in range(i-gap,-1,-gap):
		while j>=0 and svec[j]>svec[j+gap]:
		    temp        = svec[j]
		    svec[j]     = svec[j+gap]
		    svec[j+gap] = temp
		    itemp       = ivec[j]
		    ivec[j]     = ivec[j+gap]
		    ivec[j+gap] = itemp
	gap = gap / 2  # integer division needed

#    Now svec is the sorted input vec and ivec has the order
#                        svec[i] = vec[ivec[i]];
#    To convert vec to ranks, march through svec, looking for
#    duplicates, and assign the ranks back to vec.  Duplicate
#    values get the average of their ranks.

    return svec, ivec


 def arankdata(inlist):
    """\nRanks the data in inlist, dealing with ties appropritely.  Assumes
a 1D inlist.  Adapted from Gary Perlman's |Stat ranksort.

Format:  arankdata(inlist)
Returns: a list of length equal to inlist, containing rank scores\n"""

    n = len(inlist)
    svec, ivec = shellsort(inlist)
    sumranks = 0
    dupcount = 0
    for i in range(n):
	sumranks = sumranks + i
	dupcount = dupcount + 1
	if i==n-1 or svec[i] <> svec[i+1]:
	    averank = sumranks / float(dupcount) + 1
	    for j in range(i-dupcount+1,i+1):
		inlist[ivec[j]] = averank
	    sumranks = 0
	    dupcount = 0
    return inlist


 def afindwithin(data):
    """\nReturns a binary vector, 1=within-subject factor, 0=between.  Input
equals the entire data array (i.e., column 1=random factor, column -1=measured
values.

Format:  afindwithin(data)     data in |Stat format\n"""

    numfact = len(data[0])-2
    withinvec = [0]*numfact
    for col in range(1,numfact+1):
	rows = linexand(data,col,unique(colex(data,1))[0])  # get 1 level of this factor
	if len(unique(colex(rows,0))) < len(rows):   # if fewer subjects than scores on this factor
	    withinvec[col-1] = 1
    return withinvec


######################  END OF NUMERIC FUNCTION BLOCK  #####################

######################  END OF STATISTICAL FUNCTIONS  ######################

except ImportError:
 pass
